angular.module('stow.controllers')

    .controller('AccountCtrl', function ($rootScope, $scope, $state, $timeout, $stateParams, $ionicActionSheet, $ionicHistory, $cordovaFileTransfer, Camera, Picture, Account, CreditCard, BankAccount) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Account");
        }

        function init() {
            $scope.updating = false;
            $scope.showOverlay = true;
            $scope.originalPicture = $scope.displayPicture = $rootScope.user.profilePicture;
            $scope.accountInformation = {'fName': $rootScope.user.fn, 'lName': '', 'mobile': ''};

            Account.getUserData().then(function (result) {
                $rootScope.user.un = result.un;
                $scope.accountInformation.lName = result.ln;
                $scope.accountInformation.mobile = result.tel;

                if ($rootScope.user.payment.hasCreditCard && !$rootScope.user.isOwner) {
                    getCCData();
                } else if ($rootScope.user.payment.hasBankAcct && $rootScope.user.isOwner) {
                    getBankingData();
                } else {
                    $scope.showOverlay = false;
                }
            }, function (error) {
                $rootScope.showMessage(error.message, 'invalid');
            });
        }

        function getCCData() {
            CreditCard.getCustomerCreditCardData().then(function (result) {
                $scope.ccType = result.data.data.card.brand;
                $rootScope.user.payment.hasCreditCard = true;
            }, function (error) {
                $rootScope.showMessage(error.message, 'invalid');
            }).finally(function () {
                $scope.showOverlay = false;
            });
        }

        function getBankingData() {
            BankAccount.getCustomerBankingData().then(function (result) {
                $scope.bankName = result.data.data.bank_name;
            }, function (error) {
                console.log(error);
                if (error.status == 401) {
                    $rootScope.forceLogout();
                }
                $rootScope.showMessage(error.data.message, 'invalid');
            }).finally(function () {
                $scope.showOverlay = false;
            });
        }

        $scope.showAddPhotoOptions = function () {
            Picture.addImage().then(function (result) {
                $scope.showOverlay = true;
                $scope.retrievePhoto(result);
            });
        };

        $scope.retrievePhoto = function (method) {
            Camera.retrievePhoto(method, true).then(function (result) {
                console.log(result);
                $scope.displayPicture = result;
                $scope.showOverlay = false;
            }, function (error) {
                $rootScope.showMessage(error, "invalid");
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        // Save all updates to the DB
        $scope.save = function () {
            $scope.showOverlay = $scope.updating = true;
            if ($scope.pictureChanged()) {
                saveProfilePicture();
            } else {
                saveUserData();
            }
        };

        function saveProfilePicture() {
            Picture.saveProfilePicture($scope.displayPicture).then(function (result) {
                saveUserData();
            }, function (error) {
                console.log(error);
                $scope.showOverlay = false;
                $rootScope.showMessage("Error saving profile picture", "invalid");
                analytics.trackException('Failed to upload new profile picture', false);
            });
        }

        function saveUserData() {
            if ($scope.accountForm.$dirty) {
                Account.saveData($scope.accountInformation).then(function () {
                    analytics.trackEvent('Update Information', 'Information updated');
                    finish();
                }, function (error) {
                    $scope.showOverlay = false;
                    $rootScope.showMessage(error.message, 'invalid');
                    analytics.trackException('Failed to update user account info', false);
                });
            } else {
                finish();
            }
        }

        // Update all view information to be synced together
        function finish() {
            $rootScope.user.fn = $scope.accountInformation.fName;
            $rootScope.user.ln = $scope.accountInformation.lName;
            $scope.accountForm.$setPristine();
            if ($scope.pictureChanged()) {
                $rootScope.user.profilePicture = $scope.originalPicture = $scope.displayPicture;
            }
            $scope.showOverlay = $scope.updating = false;
            $rootScope.showMessage('Account Updated', 'valid');
            $ionicHistory.clearCache(); // Clears all views that may have users old data
            if ($stateParams.goBack) {
                $timeout(function () {
                    $ionicHistory.goBack();
                }, 1000);
            }
        }

        $scope.pictureChanged = function () {
            if ($scope.originalPicture !== $scope.displayPicture) {
                return true;
            } else {
                return false;
            }
        };

        $scope.deactivateAccount = function () {
            analytics.trackEvent('Deactivate Account', 'Prompting to Confirm Deactivation');
            var hideSheet = $ionicActionSheet.show({
                destructiveText: 'Deactivate Account',
                titleText: 'Are you really sure you want to DEACTIVATE your account?',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {
                    hideSheet();
                    $scope.showOverlay = true;
                    Account.deactivateAccount().then(function () {
                        $state.go('app.login');
                        $rootScope.showMessage('Account Deactivated', 'valid');
                        $timeout(function () {
                            $rootScope.user = {};
                            $scope.showOverlay = false;
                        }, 500);
                        analytics.trackEvent('Deactivate Account', 'Success');
                    }, function (error) {
                        $scope.showOverlay = false;
                        $rootScope.showMessage(error.message, 'invalid');
                    });
                }
            });
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
            init();
        });
    });