angular.module('stow.controllers')
    .controller('EditBankAccountCtrl', function ($window, $q, $http, $rootScope, $scope, $state, $ionicHistory, $stateParams, BankAccount, ManagedAccount) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Edit Bank Account");
        }

        if($rootScope.user.payment.hasBankAcct) {
            $scope.title = "Edit Bank Account";
        } else {
            $scope.title = "Add Bank Account";
        }

        $scope.bank = {
            country: 'US',
            currency: 'USD',
            account_holder_type: 'individual'
        };

        function getStripePIIToken() {
            $scope.showOverlay = true;
            Stripe.setPublishableKey($rootScope.stripeKey);
            Stripe.piiData.createToken({personal_id_number: $stateParams.managedAccountInfo.social}, createManagedAccount);
        }

        // Create a managed account with stripe for the user
        function createManagedAccount(status, response) {
            if (response.error) {
                $rootScope.showMessage(response.error.message, "invalid");
                $scope.showOverlay = false;
            } else {
                $stateParams.managedAccountInfo.ssn = response.id;
                ManagedAccount.createManagedAccount($stateParams.managedAccountInfo).then(function (result) {
                    analytics.trackEvent('Managed Account', 'Managed account created');
                    $rootScope.user.payment.hasManagedAcct = true;
                    saveUserBankingData();
                }, function (err) {
                    $scope.showOverlay = false;
                    $rootScope.showMessage(err.data.message, "invalid");
                    analytics.trackException('Failed to create managed account', false);
                });
            }
        }

        // Submit user bank information to Stripe to obtain an ID
        function getStripeBankToken() {
            var deferred = $q.defer();
            $scope.bank.routing_number = $scope.bank.routing_number.toString();
            $scope.bank.account_number = $scope.bank.account_number.toString();
            Stripe.setPublishableKey($rootScope.stripeKey);
            Stripe.bankAccount.createToken($scope.bank, function (status, response) {
                if (response.error) {
                    deferred.reject(response);
                } else {
                    deferred.resolve(response);
                }
            });
            return deferred.promise;
        }

        // Get Stripe id, then send that id to server for reference
        function saveUserBankingData() {
            $scope.showOverlay = true;
            getStripeBankToken().then(function (stripeResponse) {
                BankAccount.saveAccountBankData(stripeResponse.id).then(function (response) {
                    if ($rootScope.user.payment.hasBankAcct) {
                        $rootScope.showMessage("Connected Account Updated", "valid");
                        $ionicHistory.goBack();
                        analytics.trackEvent("Account Update", "User Updated Banking Info");
                    } else {
                        $rootScope.user.payment.hasBankAcct = true;
                        analytics.trackEvent("Account Update", "User Saved Initial Account");
                        $ionicHistory.clearCache().then(function(){
                            $rootScope.showMessage("Profile Complete. Start/finish listing your space.", "valid");
                            if($stateParams.managedAccountInfo) {
                                $ionicHistory.goBack(-2);
                            } else {
                                $ionicHistory.goBack();
                            }
                        });
                    }
                }, function (response) {
                    $scope.showOverlay = false;
                    $rootScope.showMessage(response.error.message, "invalid");
                    analytics.trackException('Failed to update banking information', false);
                })
            }, function (response) {
                $scope.showOverlay = false;
                $rootScope.showMessage(response.error.message, "invalid");
                analytics.trackException('Failed to get Stripe token', false);
            });
        }

        $scope.submit = function() {
            if($rootScope.user.payment.hasManagedAcct) {
                saveUserBankingData();
            } else {
                getStripePIIToken();
            }
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
        });
    });