angular.module('stow.controllers')
    .controller('CompleteProfileCtrl', function ($window, $q, $http, $timeout, $rootScope, $scope, $state, $stateParams, BankAccount, ManagedAccount, Rental, Picture, Camera, Account) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Complete Profile");
            analytics.trackEvent("View State", "View Entered");
        }

        if ($rootScope.user.payment.hasManagedAcct) {
            $ionicHistory.goBack();
        } else if (!$stateParams.owner) {
            $scope.owner = {
                type: 'individual',
                country: 'US',
                acceptsTos: false,
                fn: $rootScope.user.fn,
                ln: $rootScope.user.ln,
                dob: {},
                social: "",
                business_name: "",
                business_tax_id: ""
            };
        } else {
            $scope.owner = $stateParams.owner;
            $scope.fullAddress = $scope.owner.street + ", " + $scope.owner.city + ", " + $scope.owner.state + " " + $scope.owner.postal;
            $scope.dob = new Date($scope.owner.dob.year, $scope.owner.dob.month - 1, $scope.owner.dob.day);
        }

        $scope.showAddPhotoOptions = function () {
            Picture.addImage().then(function (result) {
                $scope.showOverlay = true;
                $scope.retrievePhoto(result);
            });
        };

        $scope.retrievePhoto = function (method) {
            Camera.retrievePhoto(method, true).then(function (result) {
                analytics.trackEvent("User Info", "Picture Uploaded");
                $scope.displayPicture = result;
                $scope.showOverlay = false;
            }, function (error) {
                $rootScope.showMessage(error, "invalid");
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        // iOS dates need to be broken apart to seperate variables for Stripe
        $scope.setDate = function (date) {
            date = new Date(date);
            $scope.owner.dob.day = date.getDate().toString();
            $scope.owner.dob.month = (date.getMonth() + 1).toString();
            $scope.owner.dob.year = date.getFullYear().toString();
            analytics.trackEvent("User Info", "Added Birthday");
        };

        // Typeahead function to show a list of addresses matching the user's input
        $scope.getAddressList = function (val) {
            $scope.returnedAddresses = [];
            return $http.get("https://maps.googleapis.com/maps/api/geocode/json", {
                params: {
                    address: val,
                    components: "country:US",
                    sensor: false
                }
            }).then(function (response) {
                return response.data.results.map(function (item) {
                    $scope.returnedAddresses.push(item);
                    return item.formatted_address;
                });
            });
        };

        $scope.selectAddress = function () {
            angular.forEach($scope.returnedAddresses, function (value) {
                if (value.formatted_address === $scope.fullAddress) {
                    compileAddress(value);
                }
            });
        };

        // Pull apart Google location object into our object
        function compileAddress(value) {
            $scope.fullAddress = value.formatted_address;
            var temp = {addr: {}};
            Rental.compileAddress(temp, value).then(function (result) {
                $scope.owner.street = result.addr.buildingNum + " " + result.addr.street;
                $scope.owner.city = result.addr.city;
                $scope.owner.state = result.addr.state;
                $scope.owner.postal = result.addr.zip;
                analytics.trackEvent("User Info", "Address Added");
            }, function (error) {
                alert(error);
            });
        }

        $scope.clearAddress = function () {
            $scope.owner.street = $scope.owner.city = $scope.owner.state = $scope.owner.postal = '';
        };

        $scope.validAddress = function () {
            if ($scope.owner.street && $scope.owner.city && $scope.owner.state && $scope.owner.postal) {
                return true;
            } else {
                return false;
            }
        };

        $scope.managedFormIsValid = function () {
            if ($scope.owner.acceptsTos && $scope.owner.dob && $scope.validAddress() &&
                (($scope.owner.type === 'individual' && $scope.owner.social.length === 9 ) ||
                ($scope.owner.type === 'company' && $scope.owner.business_name && $scope.owner.business_tax_id.length === 9)) &&
                ($scope.displayPicture || $rootScope.user.profilePicture.src)) {
                return true;
            } else {
                return false;
            }
        };

        function saveProfilePicture() {
            $scope.showOverlay = true;
            Picture.saveProfilePicture($scope.displayPicture).then(function (result) {
                nextStep();
            }, function (error) {
                console.log(error);
                $scope.showOverlay = false;
                $rootScope.showMessage("Error saving profile picture", "invalid");
                analytics.trackException('Failed to upload new profile picture', false);
            });
        }

        $scope.determineNext = function () {
            if ($rootScope.user.profilePicture.src) {
                nextStep();
            } else {
                saveProfilePicture();
            }
        };

        function nextStep() {
            $scope.showOverlay = true;
            if (!$rootScope.user.profilePicture.src) {
                $rootScope.user.profilePicture = $scope.displayPicture;
            }
            analytics.trackEvent("View State", "Moving to Banking");
            $state.go("app.debit-card-edit", {owner: $scope.owner});
            $timeout(function () {
                $scope.showOverlay = false;
            }, 1000);
        }

        $scope.$on("$ionicView.loaded", function () {
            $scope.showOverlay = false;
            $rootScope.checkSession();
        });
    });