angular.module('stow.controllers')
    .controller('EditCreditCardCtrl', function ($q, $timeout, $rootScope, $scope, $stateParams, $ionicHistory, CreditCard) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Edit Credit Card");
        }

        // Submit user CC info to stripe to get a Stripe ID
        function getStripeCcToken() {
            var deferred = $q.defer();
            Stripe.setPublishableKey($rootScope.stripeKey);
            Stripe.card.createToken({
                number: $scope.cc.number,
                cvc: $scope.cc.cvc,
                exp_month: $scope.cc.expiration.month,
                exp_year: $scope.cc.expiration.year
            }, function (status, response) {
                if (response.error) {
                    deferred.reject(response);
                } else {
                    deferred.resolve(response);
                }
            });
            return deferred.promise;
        }

        function handleStripeResponse(customerId, method) {
            var methodToCall;
            if (method === 'update') {
                methodToCall = CreditCard.updateCustomerCreditCardData;
            } else {
                methodToCall = CreditCard.saveCustomerCreditCardData;
            }
            methodToCall(customerId).then(function (response) {
                $scope.showOverlay = false;
                $rootScope.user.payment.hasCreditCard = true;
                var message = (method === 'update') ? "Credit Card Updated" : "Credit Card Added";
                analytics.trackEvent(message, false);
                $rootScope.showMessage(message, "valid");
                $ionicHistory.goBack();
            }, function (error) {
                if (error.status == 401) {
                    $rootScope.forceLogout();
                }
                analytics.trackException("Failed to update CC", false);
                console.log(JSON.stringify(error));
                $scope.showOverlay = false;
                $rootScope.showMessage(error.data.message, "invalid");
            });
        }

        // Obtain Stripe ID and then pass to server for reference
        $scope.submit = function () {
            $scope.showOverlay = true;
            var action = ($scope.title === "Replace Card") ? "update" : "save";
            getStripeCcToken().then(function (response) {
                handleStripeResponse(response.id, action);
            }, function (response) {
                $scope.showOverlay = false;
                $rootScope.showMessage(response.error.message, "invalid");
            });
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            if ($rootScope.checkSession()) {
                $scope.cc = {};
                $scope.title = $rootScope.user.payment.hasCreditCard ? "Replace Card" : "Add Credit Card";
            }
        });
    });