angular.module('stow.controllers')
    .controller('EditDebitCardCtrl', function ($q, $timeout, $rootScope, $scope, $stateParams, $ionicHistory, DebitCard, ManagedAccount) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Edit Debit Card");
        }

        function getStripePIIToken() {
            $scope.showOverlay = true;
            console.log("Getting PII");
            var pii_number = '';
            if ($stateParams.owner.type == 'individual') {
                pii_number = $stateParams.owner.social;
            } else {
                pii_number = $stateParams.owner.business_tax_id;
            }
            console.log(pii_number);
            Stripe.setPublishableKey($rootScope.stripeKey);
            Stripe.piiData.createToken({personal_id_number: pii_number}, createManagedAccount);
        }

        // Create a managed account with stripe for the user
        function createManagedAccount(status, response) {
            if (response.error) {
                $rootScope.showMessage(response.error.message, "invalid");
            } else {
                console.log("Got PII");
                $stateParams.owner.token = response.id;
                ManagedAccount.createManagedAccount($stateParams.owner).then(function (result) {
                    console.log(result);
                    analytics.trackEvent('Managed Account', 'Managed account created');
                    $rootScope.user.payment.hasManagedAcct = true;
                    saveUserDebitCard();
                }, function (err) {
                    console.log(err);
                    $scope.showOverlay = false;
                    $rootScope.showMessage(err.data.message, "invalid");
                    analytics.trackException('Failed to create managed account', false);
                });
            }
        }

        // Submit user Debit Card information to Stripe to obtain an ID
        function getStripeCardToken() {
            $scope.showOverlay = true;
            var deferred = $q.defer();
            Stripe.setPublishableKey($rootScope.stripeKey);
            Stripe.card.createToken({
                number: $scope.card.number,
                cvc: $scope.card.cvc,
                exp_month: $scope.card.expiration.month,
                exp_year: $scope.card.expiration.year,
                currency: $scope.card.currency
            }, function (status, response) {
                $scope.showOverlay = false;
                if (response.error) {
                    deferred.reject(response);
                } else {
                    deferred.resolve(response);
                }
            });
            return deferred.promise;
        }

        // Get Stripe id, then send that id to server for reference
        function saveUserDebitCard() {
            getStripeCardToken().then(function (stripeResponse) {
                console.log("got stripe token and going to save card");
                $scope.showOverlay = true;
                DebitCard.createDebitCardData(stripeResponse.id).then(function (response) {
                    console.log("saved debit card info");
                    $scope.showOverlay = false;
                    $rootScope.user.payment.hasDebitCard = true;
                    if ($stateParams.owner) {
                        $ionicHistory.goBack(-2);
                        $rootScope.showMessage("Profile Completed. Ready to List.", "valid");
                    } else if ($ionicHistory.backView().stateName === "app.bank-account"){
                        $state.go("app.debit-card");
                    } else {
                        $ionicHistory.goBack();
                        $rootScope.showMessage("Card Updated", "valid");
                    }
                    analytics.trackEvent('Save Data', 'Card information saved');
                }, function (response) {
                    $scope.showOverlay = false;
                    $rootScope.showMessage("Please verify the card number is a valid debit card.", "invalid");
                    analytics.trackException('Failed to update debit card information', false);
                });
            }, function (response) {
                $rootScope.showMessage(response.error.message, "invalid");
                analytics.trackException('Failed to get Stripe token', false);
            });
        }

        $scope.save = function () {
            if ($rootScope.user.payment.hasManagedAcct) {
                saveUserDebitCard();
            } else {
                console.log("Going to PII");
                getStripePIIToken();
            }
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            if ($rootScope.checkSession()) {
                $scope.card = {
                    currency: 'usd'
                };
                if ($stateParams.owner || ($rootScope.user.payment.hasBankAcct)) {
                    $scope.title = "Add Debit Card";
                } else if ($rootScope.user.payment.hasManagedAcct && $rootScope.user.payment.hasDebitCard) {
                    $scope.title = "Edit Debit Card";
                } else {
                    $ionicHistory.goBack();
                }
            }
        });

    });