var AuthenticateService = angular.module('AuthenticateService', [])

    .service('Authenticate', [
        '$http',
        '$q',
        '$rootScope',
        function ($http, $q, $rootScope) {
            return {
                login: function (loginData) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowURL + '/login',
                        data: loginData
                    };
                    console.log(req);
                    $http(req).
                        success(function (data, status, headers, config) {
                            deferred.resolve(data);
                        }).
                        error(function (data, status, headers, config) {
                            deferred.status = status;
                            deferred.reject(data);
                        });
                    return deferred.promise;
                },
                register: function(registrationData) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowURL + '/register',
                        data: registrationData
                    };
                    $http(req).
                        success(function (data, status, headers, config) {
                            deferred.resolve(data);
                        }).
                        error(function (data, status, headers, config) {
                            if(status){
                                data.status = status;
                            }
                            deferred.reject(data);
                        });
                    return deferred.promise;
                },
                verify: function(userData) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowURL + '/account/verify/',
                        data: userData
                    };
                    $http(req).
                        success(function (result) {
                            deferred.resolve(result);
                        }).
                        error(function (data, status) {
                            data.status = status;
                            deferred.reject(data);
                        });
                    return deferred.promise;
                },
                resendPIN: function(username) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowURL + '/account/pin/',
                        data: {
                            un: username
                        }
                    };
                    $http(req).
                        success(function (result) {
                            deferred.resolve(result);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                        });
                    return deferred.promise;
                },
                forgotPassword: function(username) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowURL + '/password/reset',
                        data: {
                            email: username
                        }
                    };
                    $http(req).
                        success(function (result) {
                            deferred.resolve(result);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                        });
                    return deferred.promise;
                },
                logout: function () {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/session',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token,
                            'Content-Type': 'application/json'
                        }
                    }
                    $http(req).
                        success(function (data, status, headers, config) {
                            deferred.resolve(true);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                            if(status == 401){
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                }
            }
        }]);