angular.module('stow.controllers')

    .controller('BrowseSpacesCtrl', function ($rootScope, $scope, $state, $timeout, $cordovaFileTransfer, $cordovaFile, Rental, Picture, Location, Address) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Browse");
            analytics.trackEvent("View State", "Entered Browse");
        }

        function getRentals() {
            if ($scope.page === 1) {
                $scope.spaces = [];
                if (!$scope.updating && !$scope.searchSpinner) {
                    $scope.showOverlay = true;
                }
            }
            $scope.defaultImageKeys = [];
            $scope.profileImageKeys = [];
            $scope.tempSpaces = [];
            $scope.profileImagesComplete = $scope.defaultImagesComplete = false;
            $scope.tempLength = 0;
            Rental.browseRentals($scope.page, $scope.location, $rootScope.distanceSlider.value, $rootScope.priceSlider.minValue, $rootScope.priceSlider.maxValue).then(function (results) {
                $scope.totalPages = results.data.numPagesTotal;
                $scope.tempLength = results.data.data.length;
                if ($scope.tempLength) {
                    angular.forEach(results.data.data, function ($value, $key) {
                        // Temp spaces is a duplicate of spaces but is temporary so that we can load before displaying
                        $scope.tempSpaces.push(results.data.data[$key]);
                        getProfilePicture($scope.tempSpaces.length - 1);
                        getDefaultImage($scope.tempSpaces.length - 1);
                    });
                }
            }, function (error) {
                console.log(error);
                if ("status" in error && error.status != 404) {
                    $rootScope.showMessage(error.message, 'invalid');
                    analytics.trackException('Failed to retrieve browse properties', true);
                }
                checkFinish(true);
            });
        }

        // If the space has a profile image attached to it, check if it is on the phone. If not download it
        function getProfilePicture(key) {
            if ($scope.tempSpaces[key].owner.hasOwnProperty('profile')) {
                var id = $scope.tempSpaces[key].owner.profile.picId;
                Picture.getPicture(id, id).then(function (result) {

                    $scope.tempSpaces[key].profilePictureLocation = result;
                }, function (error) {
                    console.log(error);
                }).finally(function () {
                    checkImageKey(key, "profileImage");
                });
            }
        }

        function getDefaultImage(key) {
            var foundDefaultID;
            // Find default image for space
            var len = $scope.tempSpaces[key].img.length;
            angular.forEach($scope.tempSpaces[key].img, function (value, imgKey) {
                if (!foundDefaultID) {
                    if (value.default) {
                        foundDefaultID = value.id;
                        resolve(foundDefaultID);
                    } else if (imgKey === (len - 1)) {
                        foundDefaultID = $scope.tempSpaces[key].img[0].id;
                        resolve(foundDefaultID);
                    }
                }
            });

            function resolve(foundDefaultID) {
                Picture.getPicture(foundDefaultID, foundDefaultID).then(function (result) {
                    $scope.tempSpaces[key].defaultImageLocation = result;
                    checkImageKey(key, "defaultImage");
                }, function (error) {
                    console.log(error);
                }).finally(function () {
                    checkImageKey(key, "defaultImage");
                });
            }
        }

        function checkImageKey(key, type) {
            if (type === "profileImage") {
                $scope.profileImageKeys.push(key);
                if ($scope.profileImageKeys.length === $scope.tempLength) {
                    $scope.profileImagesComplete = true;
                    checkFinish(false);
                }
            } else if (type === "defaultImage") {
                $scope.defaultImageKeys.push(key);
                if ($scope.defaultImageKeys.length === $scope.tempLength) {
                    $scope.defaultImagesComplete = true;
                    checkFinish(false);
                }
            }
        }

        function checkFinish(force) {
            if (force || ($scope.profileImagesComplete && $scope.defaultImagesComplete)) {
                $scope.profileImageKeys.sort();
                angular.forEach($scope.profileImageKeys, function ($value, $key) {
                    $scope.spaces.push($scope.tempSpaces[$value]);
                });
                if ($scope.updating || $scope.page > 1) {
                    $scope.$broadcast('scroll.infiniteScrollComplete');
                    $scope.$broadcast('scroll.refreshComplete');
                }
                $scope.showOverlay = $scope.updating = $scope.searchSpinner = $scope.loadingMore = false;
            }
        }

        $scope.loadMore = function () {
            $scope.page++;
            if ($scope.page <= $scope.totalPages) {
                $scope.loadingMore = true;
                getRentals();
                analytics.trackEvent('Scrolling', 'User loading more rentals');
            } else {
                $scope.stopInfiniteScroll = true;
            }
        };

        $scope.doRefresh = function () {
            $scope.updating = true;
            $scope.page = 1;
            $scope.stopInfiniteScroll = false;
            getRentals();
            analytics.trackEvent('Scrolling', 'Pull to refresh');
        };

        $scope.toggleSearch = function () {
            $scope.showSearch = !$scope.showSearch;
        };

        $scope.clearSearch = function (exitSearch) {
            $scope.searchLocation = "";
            $scope.searchResults = [];
            if (exitSearch) {
                $scope.toggleSearch();
            }
        };

        $scope.getAddressList = function () {
            Address.createAddressList($scope.searchLocation).then(function (results) {
                $scope.searchResults = results;
            });
        };

        $scope.selectAddress = function (address) {
            $scope.searchLocation = $scope.displayLocation = address;
            Address.selectAddress($scope.searchLocation).then(function (result) {
                $scope.searchSpinner = true;
                $scope.location.lat = result.geometry.location.lat;
                $scope.location.long = result.geometry.location.lng;
                $scope.page = 1;
                $scope.stopInfiniteScroll = false;
                getRentals();
                $scope.clearSearch(true);
            });
        };

        $scope.isUpdatingView = function () {
            if ($scope.showOverlay || $scope.updating || $scope.loadingMore || $scope.searchSpinner) {
                return true;
            } else {
                return false;
            }
        };

        $scope.$on('$ionicView.loaded', function () {
            $scope.showSearch = $scope.profileImagesComplete = $scope.defaultImagesComplete = false;
            $scope.stopInfiniteScroll = $scope.updating = false;
            $scope.showOverlay = true;
            $scope.location = {};
            $scope.page = 1;
            $scope.spaces = $scope.tempSpaces = $scope.searchResults = [];
            Location.getLocation().then(function (result) {
                $scope.location = result;
            }, function (error) {
                console.log("couldn't get location");
                console.log(error);
            }).finally(function () {
                getRentals();
            });
        });
    });