var CameraService = angular.module('CameraService', [])

    .service('Camera', [
        '$http',
        '$cordovaCamera',
        '$cordovaFile',
        '$q',
        function ($http, $cordovaCamera, $cordovaFile, $q) {
            // Function to create a unique name for the file
            function createName(name) {
                var text = "";
                var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
                for (var i = 0; i < 15; i++)
                    text += possible.charAt(Math.floor(Math.random() * possible.length));
                return text + ".jpeg";
            }

            function toDataUrl(url, callback) {
                var xhr = new XMLHttpRequest();
                xhr.responseType = 'blob';
                xhr.onload = function() {
                    var reader = new FileReader();
                    reader.onloadend = function() {
                        callback(reader.result);
                    }
                    reader.readAsDataURL(xhr.response);
                };
                xhr.open('GET', url);
                xhr.send();
            }

            // Set the $cordovaCamera options based on upload style and if an overlay is needed
            function pictureOptions(method, addPopover) {
                var source;
                if (method === 'Take Photo') {
                    source = Camera.PictureSourceType.CAMERA;
                } else if (method === 'Choose Existing') {
                    source = Camera.PictureSourceType.PHOTOLIBRARY
                }
                var options = {
                    quality: 25,
                    targetWidth: 1028,
                    targetHeight: 720,
                    destinationType: Camera.DestinationType.FILE_URI,
                    sourceType: source,
                    encodingType: Camera.EncodingType.JPEG,
                    saveToPhotoAlbum: false,
                    correctOrientation: true
                };
                if (addPopover && ionic.Platform.isIOS()) {
                    // popover is the $cordovaCamera option to have an overlay to edit an image after taking/uploading
                    // this option only plays nicely on iOS though
                    options.allowEdit = true;
                    options.popoverOptions = new CameraPopoverOptions(300, 300, 100, 100, Camera.PopoverArrowDirection.ARROW_ANY);
                }
                return options;
            }

            return {
                retrievePhoto: function (method, addPopover) {
                    var deferred = $q.defer();
                    var options = pictureOptions(method, addPopover);
                    $cordovaCamera.getPicture(options).then(function (imageData) {
                        // Seperate the file and directory path
                        if (imageData) {
                            toDataUrl(imageData, function(base64Img) {
                                var ret = {
                                    name: createName(),
                                    type: "image/jpeg",
                                    data: base64Img,
                                    src: base64Img
                                };
                                deferred.resolve(ret);
                            });
                        } else {
                            deferred.reject("Please use a picture saved to your device.");
                        }

                    }, function (error) {
                        console.error(JSON.stringify(error, null, " "));
                        deferred.reject("Unable to get photo");
                    });
                    return deferred.promise;
                }
            }
        }]);