angular.module('stow.controllers')

    .controller('LoginCtrl', function ($rootScope, $scope, $state, $http, $timeout, $cordovaFileTransfer, $ionicSlideBoxDelegate, $ionicHistory, $localStorage, Authenticate, Account, Picture) {
        if (typeof analytics !== "undefined") {
            analytics.trackView("Login/Registration");
        }

        $scope.init = function () {
            $scope.uid = 0;
            $scope.slide = 0;
            $scope.showNav = true;
            $scope.showBack = false;
            $scope.history = [];
            $scope.loginOrRegister = "Register";
            $scope.loginData = $scope.registrationData = {};
            $ionicSlideBoxDelegate.enableSlide(false);
        };

        $scope.changeSlide = function (slideNumber, goingBack) {
            if (slideNumber > 1 && !$scope.showBack) {
                $scope.showNav = false;
                $timeout(function () {
                    $scope.showBack = true;
                }, 200);
            } else if (slideNumber <= 1 && $scope.showBack) {
                $scope.showBack = false;
                $timeout(function () {
                    $scope.showNav = true;
                }, 200);
            }
            if (!goingBack) {
                $scope.history.push($scope.slide);
            }
            $scope.slide = slideNumber;
            $ionicSlideBoxDelegate.slide(slideNumber);
            $ionicSlideBoxDelegate.update();
            $scope.showOverlay = false;
        };

        $scope.toggleExperience = function () {
            if ($scope.slide === 0) {
                $scope.changeSlide(1);
                $scope.loginOrRegister = "Sign In";
                analytics.trackEvent('View State', 'Entered login view');
            } else if ($scope.slide === 1) {
                $scope.changeSlide(0);
                $scope.loginOrRegister = "Register";
                analytics.trackEvent('View State', 'Entered registration view');
            } else if ($scope.slide === 2) {
                analytics.trackEvent('View State', 'Entered resend PIN view');
            } else if ($scope.slide === 3) {
                analytics.trackEvent('View State', 'Entered verify PIN view');
            }
        };

        $scope.login = function () {
            $scope.showOverlay = true;
            Authenticate.login($scope.loginData).then(function (result) {
                console.log(result);
                if ($scope.rememberLogin) {
                    $scope.storage.savedProfile = {"un": $scope.loginData.un, "pw": $scope.loginData.pw};
                    console.log($scope.storage.savedProfile);
                } else {
                    $scope.storage.$reset();
                }
                $rootScope.user = result;
                console.log($rootScope.user);
                if ($rootScope.user.picId) {
                    var systemDirectory = cordova.file.dataDirectory;
                    window.resolveLocalFileSystemURL(systemDirectory + $rootScope.user.picId, assignImage, downloadImage);
                } else {
                    $rootScope.user.profilePicture = {'src': ''};
                    finish();
                }

            }, function (error) {
                console.log(error);
                if (error.status === 401) {
                    $rootScope.showMessage("Not quite. Check your information and try again.", "invalid");
                } else {
                    $rootScope.showMessage(error.message, "invalid");
                }
                $scope.showOverlay = false;
            });
        };

        function assignImage() {
            var img = {
                'src': cordova.file.dataDirectory + $rootScope.user.picId
            };
            $rootScope.user.profilePicture = img;
            finish();
        }

        function downloadImage() {
            Picture.getPicture($rootScope.user.picId, $rootScope.user.picId).then(function (result) {
                $rootScope.user.profilePicture = result;
            }, function (error) {
                console.log(error);
            }).finally(function () {
                finish();
            });
        }

        function finish() {
            $scope.showOverlay = false;
            if (!$rootScope.user.account.status.active) {
                $state.go('app.reactivate');
            }
            else if ($rootScope.user.account.status.active && $rootScope.user.isOwner) {
                $state.go('app.manage');
            } else if ($rootScope.user.account.status.active && !$rootScope.user.isOwner) {
                $state.go('app.browse');
            }
        }

        $scope.register = function () {
            $scope.showOverlay = true;
            Authenticate.register($scope.registrationData).then(function (result) {
                analytics.trackEvent('Register', 'Account registered');
                $scope.uid = result.id;
                $scope.changeSlide(3);
            }, function (error) {
                console.log(error);
                if (error.status === 409) {
                    $rootScope.showMessage("Email already registered. Try logging in or resetting your password.", "invalid");
                } else {
                    analytics.trackException('Registration failed', false);
                    $rootScope.showMessage(error.message, "invalid");
                }
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        $scope.verify = function () {
            $scope.showOverlay = true;
            var userData = {
                'userId': $scope.uid,
                'verificationPin': $scope.verificationPin
            };

            Authenticate.verify(userData).then(function () {
                if ($scope.loginData.un && $scope.loginData.pw) {
                    analytics.trackEvent('Verification', 'Account Verified When Registering');
                    Authenticate.login($scope.loginData).then(function (result) {
                        $rootScope.user = result;
                        $rootScope.user.profilePicture = {src: ""};
                        console.log($rootScope.user);
                        $scope.storage.savedProfile = {"un": $scope.loginData.un, "pw": $scope.loginData.pw};
                        $state.go("app.choose-experience");
                    }, function (error) {
                        console.log(error);
                        $rootScope.showMessage(error.message, "invalid");
                    }).finally(function () {
                        $scope.showOverlay = false;
                    });
                } else {
                    analytics.trackEvent('Verification', 'Account verified after resending PIN');
                    $rootScope.showMessage("Account Verified", "valid");
                    $scope.changeSlide(0);
                }
            }, function (error) {
                console.log(error);
                $scope.showOverlay = false;
                if (error.status === 400) {
                    $rootScope.showMessage("Incorrect PIN. Please verify 4-digit PIN.", "invalid");
                } else {
                    analytics.trackException('Verification failed', false);
                    $rootScope.showMessage(error.message, "invalid");
                }
            });
        };

        $scope.resendPIN = function () {
            $scope.showOverlay = true;
            Authenticate.resendPIN($scope.resendUsername).then(function (result) {
                analytics.trackEvent('Resend PIN', 'Success');
                $scope.uid = result.data.userId;
                $scope.changeSlide(3);
            }, function (error) {
                analytics.trackException('PIN Failed to Send', false);
                $rootScope.showMessage(error.message, "invalid");
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        $scope.sendResetInstructions = function () {
            $scope.showOverlay = true;
            Authenticate.forgotPassword($scope.resendUsername).then(function (result) {
                analytics.trackEvent('Reset Password Request', 'Success');
                $rootScope.showMessage("Email instructions sent", "valid");
                $scope.changeSlide(0);
            }, function (error) {
                analytics.trackException('Reset Password Request Failed', false);
                $rootScope.showMessage(error.message, "invalid");
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        $scope.$on("$ionicView.enter", function () {
            $rootScope.user = {};
            $ionicHistory.clearCache();
            $ionicHistory.clearHistory();
            delete $rootScope.lastMailbox;
            $scope.storage = $localStorage;
            if ($scope.storage.savedProfile) {
                $scope.rememberLogin = true;
                $scope.loginData.un = $scope.storage.savedProfile.un;
                $scope.loginData.pw = $scope.storage.savedProfile.pw;
            } else {
                $scope.rememberLogin = false;
            }
        });
    });