angular.module('stow.controllers')
    .controller('MessagesCtrl', function ($rootScope, $scope, $state, $stateParams, $ionicActionSheet, Messages) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Messages");
        }

        // Choose which mailbox to initially display. Can be altered when deleting a message
        if ($rootScope.lastMailbox === "Inbox" || !$rootScope.lastMailbox) {
            $scope.showInbox = true;
            $scope.inOrOut = 'Sent';
        } else if ($rootScope.lastMailbox === "Sent") {
            $scope.showInbox = false;
            $scope.inOrOut = 'Inbox';
        }
        $scope.showOverlay = true;
        $scope.page = 1;
        $scope.messages = [];
        $scope.moreData = false;
        $scope.loading = false;

        function getMessages() {
            if ($scope.showInbox) {
                Messages.getInboxMessages($scope.page).then(function (result) {
                    updateData(result);
                }, function (error) {
                    getMessagesError(error);
                });
            } else {
                Messages.getOutboxMessages($scope.page).then(function (result) {
                    updateData(result);
                }, function (error) {
                    getMessagesError(error);
                });
            }
        }

        function getMessagesError(error) {
            if (error.status == 404 && $scope.page === 1) {
                $scope.messages = "";
            } else if (error.status == 404) {
                //Stop infinite from scroll from looking for data
                $scope.moreData = true;
            } else {
                $scope.moreData = true;
                $rootScope.showMessage(error.message, 'invalid');
                console.log(error);
            }
            finish();
        }

        function updateData(result) {
            if ($scope.showInbox) {
                $rootScope.user.msg.unread = result.numUnread;
            }
            angular.forEach(result.data, function (value, key) {
                // Push all new messages to ng-repeat
                $scope.messages.push(value);
            });
            finish();
        }

        function finish() {
            $scope.$broadcast('scroll.infiniteScrollComplete');
            $scope.$broadcast('scroll.refreshComplete');
            $scope.showOverlay = $scope.loading = false;
        }

        $scope.toggleInOut = function () {
            $scope.showOverlay = true;
            $scope.page = 1;
            $scope.messages = [];
            $scope.showInbox = !$scope.showInbox;
            if ($scope.showInbox) {
                analytics.trackEvent("Toggle View", "Viewing Inbox");
                $scope.inOrOut = 'Sent';
            } else {
                analytics.trackEvent("Toggle View", "Viewing Outbox");
                $scope.inOrOut = 'Inbox';
            }
            getMessages();
        };

        $scope.loadMore = function () {
            $scope.page++;
            getMessages();
        };

        $scope.doRefresh = function () {
            $scope.messages = [];
            $scope.page = 1;
            $scope.moreData = false;
            $scope.loading = true;
            getMessages();
        };

        $scope.viewMessage = function (index) {
            // Decrement sidebar badge
            if (!$scope.messages[index].read && $scope.showInbox) {
                $rootScope.user.msg.unread--;
                $scope.messages[index].read = true;
            }
            var box = "Inbox";
            if (!$scope.showInbox) {
                box = "Sent";
                analytics.trackEvent("View State", "Going to View Outbox Message");
            } else {
                analytics.trackEvent("View State", "Going to View Inbox Message");
            }
            // lastMailbox is used to store the state of the messages view so we know how to reload it when user return
            $rootScope.lastMailbox = box;
            $state.go('app.view-message', {messageId: $scope.messages[index]._id, mailbox: box});
        };

        getMessages();

        $scope.$on('$ionicView.beforeEnter', function () {
            $rootScope.checkSession();
            if ($rootScope.deletedMessage) {
                angular.forEach($scope.messages, function (value, key) {
                    if (value._id === $rootScope.deletedMessage) {
                        $scope.messages.splice(key, 1);
                        delete $rootScope.deletedMessage;
                        console.log("message deleted");
                    }
                });
            }
        });
    });