var MessagesService = angular.module('MessagesService', [])

    .service('Messages', [
        '$window',
        '$http',
        '$rootScope',
        '$q',
        function ($window, $http, $rootScope, $q) {
            return {
                getInboxMessages: function (page) {
                    var deferred = $q.defer();
                    var sendURL = $rootScope.stowSecureURL + '/messages/inbox';
                    if (page > 1) {
                        sendURL = sendURL + '?page=' + page;
                    }
                    var req = {
                        method: 'GET',
                        url: sendURL,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            } else if (status == 404) {
                                results.status = 404;
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getInboxMessage: function (messageId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/messages/inbox/' + messageId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            } else if (status == 404) {
                                results.status = 404;
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getOutboxMessages: function (page) {
                    var deferred = $q.defer();
                    var sendURL = $rootScope.stowSecureURL + '/messages/outbox';
                    if (page > 1) {
                        sendURL = sendURL + '?page=' + page;
                    }
                    var req = {
                        method: 'GET',
                        url: sendURL,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            } else if (status == 404) {
                                results.status = 404;
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getOutboxMessage: function (messageId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/messages/outbox/' + messageId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            } else if (status == 404) {
                                results.status = 404;
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                sendMessage: function (message) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowSecureURL + '/messages/inbox',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        },
                        data: message
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results) {
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                deleteInboxMessage: function (messageId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/messages/inbox/' + messageId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                deleteOutboxMessage: function (messageId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/messages/outbox/' + messageId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                blockUser: function (id) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowSecureURL + '/messages/block',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        },
                        data: {
                            userId: id
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                unblockUser: function (id) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/messages/block/' + id,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                }
            }
        }]);