angular.module('stow.controllers')
    .controller('ViewMessageCtrl', function ($rootScope, $scope, $state, $stateParams, $ionicActionSheet, $ionicHistory, Messages, Picture) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("View Message");
        }

        $scope.showOverlay = $scope.loading = true;
        $scope.mailbox = $stateParams.mailbox;
        if ($scope.mailbox === 'Inbox') {
            Messages.getInboxMessage($stateParams.messageId).then(function (result) {
                analytics.trackEvent("View Message", "Inbox Message");
                assignResults(result);
            }, function (error) {
                analytics.trackException("Failed to get inbox message", false);
                console.log(error);
                $scope.showOverlay = false;
                $rootScope.showMessage(error.message, "invalid");
            });
        } else {
            Messages.getOutboxMessage($stateParams.messageId).then(function (result) {
                analytics.trackEvent("View Message", "Outbox Message");
                assignResults(result.msg);
            }, function (error) {
                analytics.trackException("Failed to get outbox message", false);
                console.log(error);
                $scope.showOverlay = false;
                $rootScope.showMessage(error.message, "invalid");
            });
        }

        function assignResults(message) {
            $scope.message = message;
            console.log($scope.message);
            getUserImage();
        }

        function getUserImage() {
            var picId = "";
            if ($scope.mailbox === 'Inbox') {
                if ("profile" in $scope.message.sender) {
                    picId = $scope.message.sender.profile.picId;
                }

            } else {
                if ("profile" in $scope.message.recipient) {
                    picId = $scope.message.recipient.profile.picId;
                }
            }
            if (picId) {
                Picture.getPicture(picId, picId).then(function (result) {
                    $scope.message.userPicture = result;
                }, function (error) {
                    alert("Unable to get picture");
                    console.log(error);
                }).finally(function () {
                    $scope.showOverlay = $scope.loading = false;
                });
            } else {
                $scope.showOverlay = $scope.loading = false;
            }
        }

        $scope.compose = function () {
            if (($scope.mailbox === 'Inbox' && $scope.message.sender.active) || ($scope.mailbox === 'Sent' && $scope.message.recipient.active)) {
                var id;
                if ($scope.mailbox === "Inbox") {
                    id = $scope.message.sender.id;
                } else {
                    id = $scope.message.recipient.id;
                }
                //Check is we are blocking/have been block by this user.
                if ($rootScope.user.msg.blockedBy.indexOf(id) !== -1) {
                    $rootScope.showMessage('Unable to reply. This user has blocked you.', 'invalid');
                } else if ($rootScope.user.msg.blocking.indexOf(id) !== -1) {
                    $rootScope.showMessage('Unable to reply. You have blocked this user', 'invalid');
                } else {
                    analytics.trackEvent("View State", "Replying to message");
                    $state.go('app.compose', {
                        'rentalId': $scope.message.rental.id,
                        'userId': id,
                        'msg': $scope.message.msg
                    });
                }
            } else {
                $rootScope.showMessage('This user has deactivated their Stow account.', 'invalid');
            }
        };

        $scope.deleteMessage = function () {
            var hideSheet = $ionicActionSheet.show({
                destructiveText: 'Delete Message',
                titleText: 'Are you sure you want to DELETE this message?',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {
                    hideSheet();
                    $scope.showOverlay = true;
                    if ($scope.mailbox === 'Inbox') {
                        Messages.deleteInboxMessage($stateParams.messageId).then(function (result) {
                            analytics.trackEvent("Delete Message", "Deleted Inbox Message");
                            backToMailbox();
                        }, function (error) {
                            $scope.showMessage(error.message, "invalid");
                        }).finally(function () {
                            $scope.showOverlay = false;
                        });
                    } else {
                        Messages.deleteOutboxMessage($stateParams.messageId).then(function (result) {
                            analytics.trackEvent("Delete Message", "Deleted Outbox Message");
                            backToMailbox();
                        }, function (error) {
                            $rootScope.showMessage(error.message, "invalid");
                        }).finally(function () {
                            $scope.showOverlay = false;
                        });
                    }
                }
            });
        };

        function backToMailbox() {
            $rootScope.showMessage("Message Deleted", "valid");
            $rootScope.deletedMessage = $scope.message._id;
            $ionicHistory.goBack();
        }

        $scope.blockUser = function () {

            if (($scope.mailbox === 'Inbox' && $scope.message.sender.active) || ($scope.mailbox === 'Sent' && $scope.message.recipient.active)) {
                var id;
                if ($scope.mailbox === 'Inbox') {
                    id = $scope.message.sender.id;
                } else {
                    id = $scope.message.recipient.id;
                }

                //Check to see if we are already blocking this user
                if ($rootScope.user.msg.blocking.indexOf(id) !== -1) {
                    var hideSheet;
                    analytics.trackEvent("Unblock User", "Prompted to Unblock User");
                    hideSheet = $ionicActionSheet.show({
                        titleText: 'Would you like to unblock this user?',
                        buttons: [
                            {text: 'Unblock User'},
                        ],
                        cancelText: 'Cancel',
                        buttonClicked: function (buttonIndex) {
                            hideSheet();
                            $scope.showOverlay = true;
                            Messages.unblockUser(id).then(function (result) {
                                analytics.trackEvent("Unblock User", "Success");
                                var index = $rootScope.user.msg.blocking.indexOf(id);
                                $rootScope.user.msg.blocking.splice(index);
                                $rootScope.showMessage("You have unblocked this user", "valid");
                            }, function (error) {
                                $scope.showMessage(error.message, "invalid");
                            }).finally(function () {
                                $scope.showOverlay = false;
                            });
                        }
                    });
                } else {
                    analytics.trackEvent("Block User", "Prompted to Block User");
                    hideSheet = $ionicActionSheet.show({
                        destructiveText: 'Block User',
                        titleText: 'Are you sure you want to BLOCK this user from messaging you?',
                        cancelText: 'Cancel',
                        destructiveButtonClicked: function () {
                            hideSheet();
                            $scope.showOverlay = true;
                            Messages.blockUser(id).then(function (result) {
                                analytics.trackEvent("Block User", "Blocked User");
                                $rootScope.user.msg.blocking.push(id);
                                $rootScope.showMessage("User will no longer be able to contact you", "valid");
                            }, function (error) {
                                $scope.showMessage(error.message, "invalid");
                            }).finally(function () {
                                $scope.showOverlay = false;
                            });
                        }
                    });
                }
            } else {
                $rootScope.showMessage('This user has deactivated their Stow account.', 'invalid');
            }
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            $rootScope.checkSession();
        });
    });