var PictureService = angular.module('PictureService', [])

    .service('Picture', [
        '$rootScope',
        '$http',
        '$q',
        '$cordovaFileTransfer',
        '$ionicActionSheet',
        function ($rootScope, $http, $q, $cordovaFileTransfer, $ionicActionSheet) {
            return {
                addImage: function (){
                    var deferred = $q.defer();
                    // Show the action sheet
                    $ionicActionSheet.show({
                        buttons: [
                            {text: 'Take Photo'},
                            {text: 'Choose Existing'}
                        ],
                        titleText: 'Add Photo',
                        cancelText: 'Cancel',
                        buttonClicked: function (index) {
                            if (index === 0) {
                                deferred.resolve('Take Photo');
                            } else if (index === 1) {
                                deferred.resolve('Choose Existing');
                            }
                            return true;
                        }
                    });
                    return deferred.promise;
                },
                editImage: function (){
                    var deferred = $q.defer();
                    var hideSheet = $ionicActionSheet.show({
                        destructiveText: 'Delete',
                        buttons: [
                            {text: 'Make Default'}
                        ],
                        titleText: 'Edit Photo',
                        cancelText: 'Cancel',
                        destructiveButtonClicked: function () {
                            deferred.resolve('Delete');
                            hideSheet();
                        },
                        buttonClicked: function (buttonIndex) {
                            if (buttonIndex === 0) {
                                deferred.resolve('Make Default');
                            }
                            hideSheet();
                        }
                    });
                    return deferred.promise;
                },
                getPicture: function (filename, pictureId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: encodeURI($rootScope.stowURL + '/pictures/' + pictureId)
                    };
                    $http(req).
                    success(function (data) {
                        if(data.includes("data:")) {
                            var image = {
                                name: filename + ".jpeg",
                                type: "image/jpeg",
                                data: data,
                                src: data
                            };
                            deferred.resolve(image);
                        } else {
                            deferred.resolve({src: $rootScope.stowURL + '/pictures/' + pictureId})
                        }
                    }).
                    error(function (data, status) {
                        deferred.reject(data);
                    });

                    return deferred.promise;
                },
                makeDefault: function(rentalId, pictureId){
                    var deferred = $q.defer();
                    var req = {
                        method: 'PUT',
                        url: $rootScope.stowSecureURL + '/owner/rentals/pictures/' + rentalId + '/' + pictureId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (data) {
                            deferred.resolve(data);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                            if(status == 401){
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                deletePicture: function(rentalId, pictureId){
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/owner/rentals/pictures/' + rentalId + '/' + pictureId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (data) {
                            deferred.resolve(data);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                            if(status == 401){
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                saveRentalImage: function (image, rentalID, key) {
                    console.log(image.name);
                    console.log(image.type);
                    console.log(key);
                    var defaultImg = false;
                    //If this is the rental default image send flag
                    if (key === 0) {
                        defaultImg = true;
                    }

                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowSecureURL + '/owner/rentals/pictures/' + rentalID,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token,
                            'Content-Type': 'application/json'
                        },
                        data: {file: {
                            name: image.name,
                            type: image.type,
                            data: image.data,
                            default: defaultImg
                        }}
                    };
                    console.log("sending image");
                    // console.log(req);
                    $http(req).
                    success(function (data) {
                        console.log(data);
                        deferred.resolve(data);
                    }).
                    error(function (data, status) {
                        console.log(data);
                        deferred.reject(data);
                    });
                    return deferred.promise;
                },
                saveProfilePicture: function (image){
                    var method = '';
                    // See if this is an initial upload or an edit of the profile picture
                    if ($rootScope.user.profilePicture && $rootScope.user.profilePicture.src !== "") {
                        method = 'PUT';
                    } else {
                        method = 'POST';
                    }

                    var deferred = $q.defer();
                    var req = {
                        method: method,
                        url: $rootScope.stowSecureURL + '/pictures',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token,
                            'Content-Type': 'application/json'
                        },
                        data: {file: {
                            name: image.name,
                            type: image.type,
                            data: image.data
                        }}
                    };

                    $http(req).
                    success(function (data) {
                        deferred.resolve(data);
                    }).
                    error(function (data, status) {
                        deferred.reject(data);
                    });
                    return deferred.promise;
                },
            }
        }]);