angular.module('stow.controllers')

    .controller('EditRentalCtrl', function ($rootScope, $scope, $http, $timeout, $stateParams, $ionicSlideBoxDelegate, $ionicActionSheet, $ionicPopup, $ionicHistory, $ionicScrollDelegate, $cordovaFileTransfer, Camera, Rental, Location, Picture, Address) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Edit Rental");
        }

        $scope.init = function () {
            $scope.images = [];
            // Recursive function to retrieve the rental images on load
            function getNextImage(tempImages, nextSpot, key) {
                Picture.getPicture($scope.rentalEdit.img[key].id, $scope.rentalEdit.img[key].id)
                    .then(function (pictureData) {
                        if ($scope.rentalEdit.img[key].default) {
                            tempImages[0] = pictureData;
                            tempImages[0].id = $scope.rentalEdit.img[key].id;
                        } else if (key <= 2) {
                            tempImages[nextSpot] = pictureData;
                            tempImages[nextSpot].id = $scope.rentalEdit.img[key].id;
                            nextSpot++;
                        }
                        // Hide the overlay after the last image has been added to the view
                        if (key === ($scope.rentalEdit.img.length - 1)) {
                            $scope.images = angular.copy(tempImages);
                            $scope.changeSlide(0);
                        } else {
                            key++;
                            getNextImage(tempImages, nextSpot, key);
                        }
                    }, function (error) {
                        $rootScope.showMessage("Unable to get rental picture" + error.message, "invalid");
                    });
            }

            $ionicSlideBoxDelegate.enableSlide(false);
            $scope.showOverlay = true;
            $scope.title = 'Edit Rental';
            $scope.history = [];
            $scope.changes = {};
            $scope.imageChanges = [];
            $scope.showUpdateButton = false;
            $scope.fullAddress = '';
            $scope.imageIndexPosition = 0;
            // This object is a temporary array to hold objects for the address typeahead
            $scope.returnedAddresses = [];
            Rental.getOwnerRental($stateParams.rentalId).then(function (result) {
                // Create two copies of the results so that we can compare for changes
                $scope.rentalOriginal = angular.copy(result);
                $scope.rentalEdit = angular.copy(result);
                // Compile the full address to be displayed
                $scope.fullAddress =
                    $scope.rentalEdit.addr.buildingNum + ' ' + $scope.rentalEdit.addr.street + ', ' +
                    $scope.rentalEdit.addr.city + ', ' + $scope.rentalEdit.addr.state + ' ' +
                    $scope.rentalEdit.addr.zip + ', ' + $scope.rentalEdit.addr.country;
                if ($scope.rentalEdit.img.length) {
                    // Start getting images recursively. Send temporary array to cache images while retrieving, an initial index of where to add the next
                    // non default image and the 0 index of the rental img array
                    getNextImage([], 1, 0);
                } else {
                    $scope.changeSlide(0);
                }
            }, function (error) {
                $rootScope.showMessage(error.message, 'invalid');
                $scope.changeSlide(0);
            });
        };

        // Change the slide within the ion-slide-box
        $scope.changeSlide = function (slideNumber, fromBack) {
            $scope.showOverlay = true;
            if (!fromBack) {
                $scope.history.push($scope.slide);
            }
            if (slideNumber === 0) {
                analytics.trackEvent("View State", "Viewing Default View");
                $scope.title = "Edit Rental";
            } else if (slideNumber === 1) {
                analytics.trackEvent("View State", "Viewing Details");
                $scope.title = "Details";
            } else if (slideNumber === 2) {
                analytics.trackEvent("View State", "Viewing Dimensions");
                $scope.title = "Dimensions";
            }
            $scope.slide = slideNumber;
            $ionicSlideBoxDelegate.slide(slideNumber);
            $ionicSlideBoxDelegate.update();
            $ionicScrollDelegate.scrollTop();
            $scope.showOverlay = false;
        };

        // Pop item from view history
        $scope.checkBack = function () {
            if ($scope.slide === 0) {
                if ($scope.showUpdateButton) {
                    analytics.trackEvent("Leave View", "Trying to Leave Without Saving");
                    var confirmPopup = $ionicPopup.confirm({
                        title: 'Unsaved Changes',
                        template: 'Are you sure you want to go back? You will lose any unsaved changes.'
                    });
                    confirmPopup.then(function (res) {
                        if (res) {
                            analytics.trackEvent("Leave View", "Left Without Saving");
                            $ionicHistory.goBack();
                        }
                    });
                } else {
                    $ionicHistory.goBack();
                }
            } else {
                $scope.changeSlide($scope.history.pop(), true);
            }
        };

        // Pull apart Google location object into our object
        $scope.compileAddress = function (value) {
            $scope.clearAddress(true);
            $scope.fullAddress = value.formatted_address;
            Rental.compileAddress($scope.rentalEdit, value).then(function (result) {
                $scope.rentalEdit = result;
                $scope.changeRental('addr');
                $scope.changeRental('geo');
            }, function (error) {
                alert(error);
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        // Get current location using the phone's location services
        $scope.getCurrentAddressLocation = function () {
            $rootScope.showOverlay = true;
            Location.getLocation().then(function (position) {
                Location.getCurrentAddress(position).then(function (result) {
                    analytics.trackEvent("Change Information", "Changed to Current Address");
                    $scope.compileAddress(result);
                }, function (error) {
                    $rootScope.showMessage(error, "invalid");
                }).finally(function () {
                    $scope.showOverlay = false;
                });
            });
        };

        $scope.getAddressList = function (val) {
            return Address.createAddressList(val);
        };

        $scope.selectAddress = function () {
            Address.selectAddress($scope.fullAddress).then(function (result) {
                $scope.compileAddress(result);
            });
        };

        $scope.clearAddress = function (forceClear) {
            if (forceClear || !$scope.fullAddress) {
                $scope.rentalEdit.addr = Address.getBlankAddress();
                $scope.changeRental('addr');
            }
        };

        $scope.validAddress = function () {
            if ($scope.rentalEdit && "addr" in $scope.rentalEdit) {
                return Address.addressValid($scope.rentalEdit.addr);
            }
        };

        $scope.chooseOptions = function (index) {
            if (index in $scope.images && $scope.images[index]) {
                $scope.showEditOptions(index);
            } else {
                $scope.showAddPhotoOptions(index);
            }
        };

        $scope.showAddPhotoOptions = function (index) {
            Picture.addImage().then(function (result) {
                $scope.showOverlay = true;
                $scope.retrievePhoto(result, index);
            });
        };

        $scope.retrievePhoto = function (method, index) {
            Camera.retrievePhoto(method, false).then(function (result) {
                if (!$scope.images[0]) {

                    $scope.images[0] = result;
                } else {
                    $scope.images[index] = result;
                }

                $scope.showOverlay = false;
                analytics.trackEvent("Change Information", "Added New Image");
                $scope.imageChanges.push({action: 'POST', src: result.src, key: index});
                $scope.showUpdateButton = $scope.imageChanges.length > 0;
            }, function (error) {
                console.log(error);
                $rootScope.showMessage("Error uploading image", "invalid");
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        // Edit photo in view
        $scope.showEditOptions = function (index) {
            Picture.editImage().then(function (result) {
                if (result === 'Delete') {
                    // Search the changes array for any actions on this image
                    angular.forEach($scope.imageChanges, function (value, key) {
                        // Remove the image POST if it is a new image you are deleting
                        if ($scope.imageChanges[key].action === 'POST' && ($scope.imageChanges[key].src === $scope.images[index].src)) {
                            $scope.imageChanges.splice(key, 1);
                        } else if ($scope.imageChanges[key].id === $scope.images[index.id]) {
                            $scope.imageChanges.splice(key, 1);
                        }
                    });
                    // If the image has an ID, we need to remove it from the DB
                    if ($scope.images[index].id) {
                        analytics.trackEvent("Change Information", "Deleted Image");
                        $scope.imageChanges.push({action: 'DELETE', id: $scope.images[index].id, key: index});
                    }
                    // Delete the image from the view
                    $scope.images[index] = '';

                } else if (result === 'Make Default') {
                    analytics.trackEvent("Change Information", "Changed Default Image");
                    //If the current default image is a new image, we need to remove the default flag when it is sent to the DB
                    if ($scope.images[0] && $scope.images[0].hasOwnProperty("default")) {
                        delete $scope.images[0].default;
                    }
                    else {
                        // Remove a PUT request on the old default image if it exists
                        angular.forEach($scope.imageChanges, function (value, key) {
                            if ($scope.imageChanges[key].action === 'PUT' && ($scope.imageChanges[key].id === $scope.images[0].id)) {
                                $scope.imageChanges.splice(key, 1);
                            }
                        });
                    }
                    // Swap default image
                    if ($scope.images[0]) {
                        var temp = $scope.images[0];
                        $scope.images[0] = $scope.images[index];
                        $scope.images[index] = temp;
                    } else {
                        $scope.images[0] = $scope.images[index];
                        $scope.images[index] = "";
                    }

                    // If the new default image exists in the DB we need to create a PUT query
                    if ($scope.images[0].id) {
                        $scope.imageChanges.push({action: 'PUT', id: $scope.images[0].id, key: 0});
                    }
                    // If it is a new image, we need to create and set the default flag
                    else {
                        $scope.images[0].default = "true";
                    }
                }
                $scope.showUpdateButton = $scope.imageChanges.length > 0;
            });
        };

        $scope.changeRental = function (key) {
            analytics.trackEvent("Change Information", "Changed Rental Information", "", key);
            var found = false;
            // If the value at the index is an object, traverse it and check for changes
            if (typeof $scope.rentalOriginal[key] === 'object') {
                angular.forEach($scope.rentalOriginal[key], function (value, innerKey) {
                    if (!found) {
                        if ($scope.rentalEdit[key][innerKey] != value) {
                            $scope.changes[key] = $scope.rentalEdit[key];
                            found = true;
                        }
                    }
                });
            }
            // Value can be compared directly
            else if ($scope.rentalOriginal[key] != $scope.rentalEdit[key]) {
                $scope.changes[key] = $scope.rentalEdit[key];
                found = true;
            }
            // Remove a key in the changes if no change was detected
            if (!found) {
                if ($scope.changes.hasOwnProperty(key)) {
                    delete $scope.changes[key];
                }
            }
            if (key === 'rate') {
                $scope.rentalEdit.rate.d = Math.ceil($scope.rentalEdit.rate.m / 30);
                $scope.rentalEdit.rate.w = Math.ceil($scope.rentalEdit.rate.m / 4);
            }
            // Check if there is anything in the changes object
            $scope.showUpdateButton = Object.keys($scope.changes).length > 0;
        };

        $scope.update = function () {
            // Check if any rental data is missing
            if (!$scope.rentalEdit.title) {
                $rootScope.showMessage("Please provide a title for your space.", "invalid");
            } else if (!$scope.rentalEdit.desc) {
                $rootScope.showMessage("Please provide a description of your space.", "invalid");
            } else if (!$scope.rentalEdit.rate.d || !$scope.rentalEdit.rate.w || !$scope.rentalEdit.rate.m) {
                $rootScope.showMessage("Please provide pricing rates for your space.", "invalid");
            } else if ($scope.rentalEdit.dim.l === null || $scope.rentalEdit.dim.w === null || $scope.rentalEdit.dim.h === null) {
                $rootScope.showMessage("Please provide dimensions for your space.", "invalid");
            } else if (!$scope.rentalEdit.addr.buildingNum || !$scope.rentalEdit.addr.street || !$scope.rentalEdit.addr.state || !$scope.rentalEdit.addr.city || !$scope.rentalEdit.addr.zip) {
                $rootScope.showMessage("Please provide a valid address for your space.", "invalid");
            } else if (!$scope.images[0].src) {
                $rootScope.showMessage("Please provide a default image for your space.", "invalid");
            } else {
                $scope.showOverlay = true;
                if (Object.keys($scope.changes).length > 0) {
                    // Only works when availability is string, not boolean...
                    if ("avail" in $scope.changes) {
                        $scope.changes.avail = $scope.changes.avail.toString();
                    }
                    Rental.updateRental($stateParams.rentalId, $scope.changes).then(function () {
                        analytics.trackEvent("Save Updates", "Update Rental Data");
                        if ($scope.imageChanges.length > 0) {
                            submitImageChanges();
                        } else {
                            resetData();
                        }
                    }, function (error) {
                        $scope.showOverlay = false;
                        $rootScope.showMessage("Unable to update Rental." + error.message, "invalid");
                        console.log(error);
                    });
                } else if ($scope.imageChanges.length > 0) {
                    submitImageChanges();
                }
            }
        };

        function submitImageChanges() {
            var pictureID = $scope.imageChanges[$scope.imageIndexPosition].id;

            if ($scope.imageChanges[$scope.imageIndexPosition].action === 'POST') {
                Picture.saveRentalImage($scope.images[$scope.imageChanges[$scope.imageIndexPosition].key], $stateParams.rentalId, $scope.imageChanges[$scope.imageIndexPosition].key).then(function (result) {
                    analytics.trackEvent("Save Updates", "Saved New Image");
                    nextImageChange();
                }, function (error) {
                    $scope.showOverlay = false;
                    $rootScope.showMessage(error.message + '. Please try again.', 'invalid');
                });
            } else if ($scope.imageChanges[$scope.imageIndexPosition].action === 'PUT') {
                Picture.makeDefault($stateParams.rentalId, pictureID).then(function (result) {
                    analytics.trackEvent("Save Updates", "Saved New Default Image");
                    nextImageChange();
                }, function (error) {
                    $scope.showOverlay = false;
                    analytics.trackException("Failed to Update Default Image", false);
                    $rootScope.showMessage(error.message + '. Please try again.', 'invalid');
                });
            } else if ($scope.imageChanges[$scope.imageIndexPosition].action === 'DELETE') {
                Picture.deletePicture($stateParams.rentalId, pictureID).then(function () {
                    analytics.trackEvent("Save Updates", "Deleted Image");
                    nextImageChange();
                }, function (error) {
                    $scope.showOverlay = false;
                    analytics.trackException("Failed to Delete Image", false);
                    $rootScope.showMessage(error.message + '. Please try again.', 'invalid');
                });
            }
        }

        function nextImageChange() {
            $scope.imageIndexPosition++;
            if ($scope.imageIndexPosition <= ($scope.imageChanges.length - 1)) {
                submitImageChanges();
            } else {
                resetData();
            }
        }

        function resetData() {
            $rootScope.showMessage("Rental Updated", "valid");
            $scope.init();
        }

        $scope.deleteRental = function () {
            analytics.trackEvent("Delete Rental", "Attempting to Delete Rental");
            var hideSheet = $ionicActionSheet.show({
                destructiveText: 'Delete Rental',
                titleText: 'Are you really sure you want to DELETE this rental?',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {
                    hideSheet();
                    $scope.showOverlay = true;
                    Rental.deleteRental($scope.rentalEdit._id).then(function () {
                        analytics.trackEvent("Delete Rental", "Deleted Rental");
                        $ionicHistory.goBack();
                        $rootScope.showMessage("Rental Deleted", "valid");
                    }, function (error) {
                        console.log(error);
                        $rootScope.showMessage(error.message, "invalid");
                    }).finally(function () {
                        $scope.showOverlay = false;
                    });
                }
            });
        };

        $scope.$on('$ionicView.beforeEnter', function () {
            $rootScope.checkSession();
        });
    });