angular.module('stow.controllers')

    .controller('ListRentalCtrl', function ($rootScope, $scope, $state, $http, $cordovaFileTransfer, $ionicPopup, $ionicSlideBoxDelegate, $ionicHistory, $ionicScrollDelegate, Camera, Location, Rental, Picture, Address) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Add Rental");
        }

        $scope.title = 'Add New Rental';
        $scope.addressSelected = false;
        $scope.images = [];
        $scope.fullAddress = '';
        $scope.rentalData = {
            avail: false,
            type: '',
            geo: [],
            title: '',
            addr: {buildingNum: '', apartmentNum: '', street: '', city: '', state: '', country: '', zip: ''},
            desc: '',
            rate: {'m': '', 'w': '', 'd': ''},
            dim: {'h': '', 'w': '', 'l': ''}
        };

        // This object is a temporary array to hold objects for the address typeahead
        $scope.returnedAddresses = [];

        $scope.init = function () {
            $ionicSlideBoxDelegate.enableSlide(false);
            $scope.history = [];
            $scope.changeSlide(0, true);
        };

        // Change the slide within the ion-slide-box
        $scope.changeSlide = function (slideNumber, fromBack) {
            $scope.showOverlay = true;
            if (!fromBack) {
                $scope.history.push($scope.slide);
            }
            if (slideNumber === 0) {
                $scope.title = "Add New Rental";
            } else if (slideNumber === 1) {
                $scope.title = "Details";
                analytics.trackEvent('View State', 'Entered details view');
            } else if (slideNumber === 2) {
                $scope.title = "Dimensions";
                analytics.trackEvent('View State', 'Entered dimensions view');
            }
            $scope.slide = slideNumber;
            $ionicSlideBoxDelegate.slide(slideNumber);
            $ionicSlideBoxDelegate.update();
            $ionicScrollDelegate.scrollTop();
            $scope.showOverlay = false;
        };

        function userHasEnteredInformation() {
            if ($scope.rentalData.title || $scope.rentalData.avail || $scope.rentalData.desc || $scope.rentalData.rate.d ||
                $scope.rentalData.rate.w || $scope.rentalData.rate.m || $scope.rentalData.dim.l.toString().length ||
                $scope.rentalData.dim.w.toString().length || $scope.rentalData.dim.h.toString().length ||
                $scope.rentalData.addr.buildingNum || $scope.rentalData.addr.street || $scope.rentalData.addr.state ||
                $scope.rentalData.addr.city || $scope.rentalData.addr.zip || $scope.images[0]) {
                return true;
            } else {
                return false;
            }
        }

        // Pop item from view history
        $scope.checkBack = function () {
            if ($scope.slide === 0 && userHasEnteredInformation()) {
                analytics.trackEvent("View State", "Prompting user they will lose their data");
                var confirmPopup = $ionicPopup.confirm({
                    title: 'Unsaved Changes',
                    template: 'Are you sure you want to go back? You will lose any unsaved changes.'
                });
                confirmPopup.then(function (res) {
                    if (res) {
                        analytics.trackEvent("View State", "User decided not to save and go back");
                        $ionicHistory.goBack();
                    } else {
                        analytics.trackEvent("View State", "User declined to go back");
                    }
                });
            } else if ($scope.slide === 0) {
                $ionicHistory.goBack();
            } else {
                $scope.changeSlide($scope.history.pop(), true);
            }
        };

        $scope.chooseOptions = function (index) {
            if (index in $scope.images) {
                $scope.showEditOptions(index);
            } else {
                $scope.showAddPhotoOptions(index);
            }
        };

        // Show action sheet to choose image source
        $scope.showAddPhotoOptions = function (index) {
            Picture.addImage().then(function (result) {
                $scope.showOverlay = true;
                retrievePhoto(result, index);
            });
        };

        // Get user photo and assign it to UI element
        function retrievePhoto(method, index) {
            Camera.retrievePhoto(method, false).then(function (result) {
                if (!$scope.images[0]) {
                    $scope.images[0] = result;
                } else {
                    $scope.images[index] = result;
                }
                analytics.trackEvent('Add Rental Data', 'Uploaded photo');
            }, function (error) {
                console.log(error);
                $rootScope.showMessage("Error uploading image", "invalid");
            }).finally(function () {
                $scope.showOverlay = false;
            });
        }

        // Edit photo in view
        $scope.showEditOptions = function (index) {
            Picture.editImage().then(function (result) {
                if (result === 'Delete') {
                    $scope.images.splice(index, 1);
                } else if (result === 'Make Default') {
                    var temp = $scope.images[0];
                    $scope.images[0] = $scope.images[index];
                    $scope.images[index] = temp;
                }
            });
        };

        // Get current location using the phone's location services
        $scope.getCurrentAddressLocation = function () {
            $scope.showOverlay = true;
            Location.getLocation().then(function (position) {
                Location.getCurrentAddress(position).then(function (result) {
                    analytics.trackEvent('Add Rental Data', 'Used current location');
                    compileAddress(result);
                }, function (error) {
                    $rootScope.showMessage(error, "invalid");
                }).finally(function () {
                    $scope.showOverlay = false;
                });
            });
        };

        $scope.getAddressList = function (val) {
            return Address.createAddressList(val);
        };

        $scope.selectAddress = function () {
            Address.selectAddress($scope.fullAddress).then(function (result) {
                compileAddress(result);
            });
        };

        // Pull apart Google location object into our object
        function compileAddress(value) {
            $scope.fullAddress = value.formatted_address;
            Rental.compileAddress($scope.rentalData, value).then(function (result) {
                $scope.rentalData = result;
            }, function (error) {
                alert(error);
            });
        }

        $scope.clearAddress = function () {
            $scope.rentalData.addr = Address.getBlankAddress();
        };

        $scope.validAddress = function () {
            if ($scope.rentalData && "addr" in $scope.rentalData) {
                return Address.addressValid($scope.rentalData.addr);
            }
        };

        $scope.calculateRates = function () {
            $scope.rentalData.rate.d = Math.ceil($scope.rentalData.rate.m / 30);
            $scope.rentalData.rate.w = Math.ceil($scope.rentalData.rate.m / 4);
            console.log($scope.rentalData.rate);
        };

        $scope.submit = function () {
            $scope.showOverlay = true;
            Rental.saveRental($scope.rentalData).then(function (result) {
                analytics.trackEvent('Save Rental', 'Saved rental data');
                $scope.newID = result.id;
                $scope.addKey = 0;
                $scope.saveImages();
            }, function (error) {
                $scope.showOverlay = false;
                console.log(error);
                $rootScope.showMessage(error.message, "invalid");
            });
        };

        $scope.saveImages = function () {
            if ($scope.addKey <= 2 && $scope.images[$scope.addKey]) {
                Picture.saveRentalImage($scope.images[$scope.addKey], $scope.newID, $scope.addKey).then(function (result) {
                    $scope.addKey++;
                    $scope.saveImages();
                }, function (error) {
                    $scope.showOverlay = false;
                    $rootScope.showMessage(error.message + '. Please try again.', 'invalid');
                });
            } else if ($scope.addKey < 2) {
                $scope.addKey++;
                $scope.saveImages();
            } else {
                analytics.trackEvent('Save Rental', 'Success');
                $scope.showOverlay = false;
                $rootScope.showMessage("Rental Added", "valid");
                $state.go('app.manage');
            }
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
        });
    });