var SpaceService = angular.module('RentalService', [])

    .service('Rental', [
        '$http',
        '$rootScope',
        '$q',
        function ($http, $rootScope, $q) {
            return {
                browseRentals: function (pageNumber, location, distance, minRate, maxRate) {
                    var req = {
                        method: 'GET',
                    };
                    if(!distance) {
                        distance = 1000;
                    }
                    if(!location.long) {
                        location.long = "-105.1376919";;
                        location.lat = "40.5566373";
                    }
                    if (location.lat && location.long) {
                        req.url = $rootScope.stowURL + '/rentals?page=' + pageNumber + '&lat=' + location.lat + '&long=' + location.long + '&distance=' + distance + '&minRate=' + minRate + '&maxRate=' + maxRate;
                    }
                    console.log(req);
                    return $http(req);
                },
                getUserRentals: function () {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/owner/rentals',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            } else if (status == 404) {
                                results.status = 404;
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getOwnerRental: function (id) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/owner/rentals/' + id,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                getRental: function (id) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowURL + '/rentals/' + id
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results) {
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                saveRental: function (spaceData) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowSecureURL + '/owner/rentals',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token,
                            'Content-Type': 'application/json'
                        },
                        data: spaceData
                    };
                    $http(req).
                        success(function (data) {
                            deferred.resolve(data);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                updateRental: function (id, spaceData) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'PUT',
                        url: $rootScope.stowSecureURL + '/owner/rentals/' + id,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        },
                        data: spaceData
                    };
                    $http(req).
                        success(function (data) {
                            deferred.resolve(data);
                        }).
                        error(function (data, status) {
                            deferred.reject(data);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                deleteRental: function (id) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/owner/rentals/' + id,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                compileAddress: function (spaceObj, addrObj) {
                    var deferred = $q.defer();
                    spaceObj.addr = {
                        buildingNum: '',
                        apartmentNum: '',
                        street: '',
                        city: '',
                        state: '',
                        country: '',
                        zip: ''
                    };
                    angular.forEach(addrObj.address_components, function (val) {
                        switch (val.types[0]) {
                            case 'street_number':
                                spaceObj.addr.buildingNum = val.short_name;
                                break;
                            case 'route':
                                spaceObj.addr.street = val.short_name;
                                break;
                            case 'locality':
                                spaceObj.addr.city = val.short_name;
                                break;
                            case 'administrative_area_level_1':
                                spaceObj.addr.state = val.short_name;
                                break;
                            case 'country':
                                spaceObj.addr.country = val.short_name;
                                break;
                            case 'postal_code':
                                spaceObj.addr.zip = val.short_name;
                                break;
                        }
                    });
                    spaceObj.geo = [];
                    spaceObj.geo.push(addrObj.geometry.location.lng);
                    spaceObj.geo.push(addrObj.geometry.location.lat);
                    deferred.resolve(spaceObj);
                    return deferred.promise;
                }
            }
        }]);