angular.module('stow.controllers')

    .controller('ViewRentalCtrl', function ($rootScope, $scope, $state, $http, $stateParams, $timeout, $ionicHistory, $ionicActionSheet, $ionicSlideBoxDelegate, $cordovaFileTransfer, $cordovaFile, $cordovaSocialSharing, Rental, Picture) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("View Rental");
        }

        $scope.showOverlay = true;
        $scope.currentImageKey = 0;
        $scope.nextSpot = 1;
        $scope.images = [];
        $scope.interval = 5000;

        // Get the rental details
        Rental.getRental($stateParams.rentalId).then(function (result) {
            analytics.trackEvent("View State", "Viewing rental");
            $scope.rental = result;
            $rootScope.viewTitle = $scope.rental.title;
            checkProfileImage();
        }, function (error) {
            analytics.trackException("Failed to get rental", false);
            $rootScope.showMessage(error.message, "invalid");
            console.log(error);
            $scope.showOverlay = false;
        });

        // Check to see if a profile img is returned with the rental
        function checkProfileImage() {
            if ($scope.rental.owner.hasOwnProperty('profile')) {
                $scope.currentProfileID = $scope.rental.owner.profile.picId;
                var filename = $rootScope.deviceImageDirectory + $scope.currentProfileID + $rootScope.pictureExtension;
                window.resolveLocalFileSystemURL(filename, assignProfileImage, downloadProfileImage);
            } else {
                checkImages();
            }
        }

        // Image was found on the phone so just assign it to be the profile picture
        function assignProfileImage(fileEntry) {
            var img = {'src': fileEntry.nativeURL};
            $scope.rental.profilePictureLocation = img;
            checkImages();
        }

        // Image wasn't found on phone so we need to download it from the server
        function downloadProfileImage() {
            Picture.getPicture($scope.currentProfileID, $scope.currentProfileID).then(function (result) {
                $scope.rental.profilePictureLocation = result;
                checkImages();
            }, function (error) {
                console.log(error);
                checkImages();
            });
        }

        // Check to see if a profile img is returned with the rental
        function checkImages() {
            if ($scope.currentImageKey <= ($scope.rental.img.length - 1)) {
                // Check where to place image in display
                if ($scope.rental.img[$scope.currentImageKey].default) {
                    $scope.settingImageIndex = 0;
                } else {
                    $scope.settingImageIndex = $scope.nextSpot;
                    $scope.nextSpot++;
                }
                var filename = $rootScope.deviceImageDirectory + $scope.rental.img[$scope.currentImageKey].id + $rootScope.pictureExtension;
                window.resolveLocalFileSystemURL(filename, assignImage, downloadImage);
            } else {
                finish();
            }
        }

        // Image was found on the phone so just assign it to be the default image
        function assignImage(fileEntry) {
            $scope.images[$scope.settingImageIndex] = {'src': fileEntry.nativeURL};
            checkImages($scope.currentImageKey++);
        }

        // Image wasn't found on phone so we need to download it from the server
        function downloadImage() {
            Picture.getPicture($scope.rental.img[$scope.currentImageKey].id, $scope.rental.img[$scope.currentImageKey].id).then(function (result) {
                $scope.images[$scope.settingImageIndex] = result;
                checkImages($scope.currentImageKey++);
            }, function (error) {
                console.log(error);
                checkImages($scope.currentImageKey++);
            });
        }

        function finish() {
            $scope.showOverlay = false;
            $ionicSlideBoxDelegate.slide(0);
            $ionicSlideBoxDelegate.update();
            $ionicSlideBoxDelegate.start();
            $scope.$apply();
        }

        function showLoginSheet() {
            var hideSheet = $ionicActionSheet.show({
                buttons: [
                    {text: 'Sign In/Register'}
                ],
                titleText: 'Sign In/Register to continue',
                cancelText: 'Cancel',
                buttonClicked: function () {
                    analytics.trackEvent("View State", "Going to login from action sheet");
                    $state.go("app.login");
                    hideSheet();
                }
            });
        }

        $scope.share = function (platform) {
            var message = "Look at this space on Stow.\n\r\n\rApp Store: https://goo.gl/xZnRoq\n\r\n\rPlay Store: https://goo.gl/i2TP0V";
            var image = $scope.images[0].src;

            if (platform === "facebook") {
                analytics.trackEvent("Sharing", "Click to share via Facebook");
                $cordovaSocialSharing.shareViaFacebookWithPasteMessageHint(message, image, null, "Save time and paste Stow's message from your clipboard.");
            } else if (platform === "twitter") {
                analytics.trackEvent("Sharing", "Click to share via Twitter");
                message = "Look at this space on Stow.\n\r\n\rDownload the App: https://stow.io";
                $cordovaSocialSharing.shareViaTwitter(message, image);
            } else {
                analytics.trackEvent("Sharing", "Click to share via another channel");
                $cordovaSocialSharing.share(message, "Stow Listing", image);
            }
        };

        $scope.requestRental = function () {
            if (!$rootScope.user.token) {
                analytics.trackEvent("Request Rental", "Attempting to request without login");
                showLoginSheet();
            } else {
                $state.transitionTo("app.request", {"rental": $scope.rental});
            }
        };

        $scope.composeMessage = function () {
            if (!$rootScope.user.token) {
                analytics.trackEvent("Message Owner", "Trying to message before login");
                showLoginSheet();
            } else {
                analytics.trackEvent("Message Owner", "Going to message owner");
                var blank = {sub: "", body: ""};
                $state.go("app.compose", {rentalId: $scope.rental._id, userId: $scope.rental.owner._id, msg: blank});
            }
        };

        $scope.slideHasChanged = function (index) {
            $scope.slideIndex = index;
            if (($ionicSlideBoxDelegate.count() - 1 ) == index) {
                $timeout(function () {
                    $ionicSlideBoxDelegate.slide(0);
                }, $scope.interval);
            }
        };
    });