angular.module('stow.controllers')

    .controller('MyReservationsCtrl', function ($rootScope, $scope, $state, Reservation) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("My Reservations");
        }

        $scope.showOverlay = $scope.loading = true;
        $scope.filters = [];

        $scope.getReservations = function (refreshOccurring) {
            $scope.loading = true;
            $scope.pendingReservations = [];
            $scope.confirmedReservations = [];
            $scope.activeReservations = [];
            $scope.completedReservations = [];
            $scope.canceledReservations = [];
            $scope.declinedReservations = [];
            $scope.expiredReservations = [];

            Reservation.getUserReservations().then(function (result) {
                $scope.hasReservations = true;
                $scope.refreshOccurring = refreshOccurring;
                angular.forEach(result, function (value) {
                    switch (value.status.current.type) {
                        case "pending":
                            $scope.pendingReservations.push(value);
                            checkIndex("pending");
                            break;
                        case "confirmed":
                            $scope.confirmedReservations.push(value);
                            checkIndex("confirmed");
                            break;
                        case "active":
                            $scope.activeReservations.push(value);
                            checkIndex("active");
                            break;
                        case "completed":
                            $scope.completedReservations.push(value);
                            break;
                        case "canceled":
                            $scope.canceledReservations.push(value);
                            break;
                        case "declined":
                            $scope.declinedReservations.push(value);
                            break;
                        case "expired":
                            $scope.expiredReservations.push(value);
                            break;
                    }
                });
            }, function (error) {
                console.log(error);
                if (error.status == 404) {
                    $scope.hasReservations = false;
                } else {
                    $rootScope.showMessage(error.message, "invalid")
                }
            }).finally(function () {
                $scope.$broadcast('scroll.refreshComplete');
                $scope.showOverlay = $scope.loading = false;
            });
        };

        function checkIndex(index) {
            if (!$scope.refreshOccurring) {
                if ($scope.filters.indexOf(index) === -1) {
                    $scope.filters.push(index);
                }
            }
        }

        $scope.inFilters = function (reservationType) {
            if ($scope.filters.indexOf(reservationType) === -1) {
                return false;
            } else {
                return true;
            }
        };

        $scope.viewReservation = function (id) {
            $rootScope.user.isOwner ? $state.go('app.owner-reservation', {reservationId: id}) : $state.go('app.renter-reservation', {reservationId: id});
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
            if ($scope.filters.length) {
                console.log("Filters exist");
                console.log($scope.filters);
                $scope.getReservations(true);
            } else {
                $scope.getReservations(false);
            }
        });
    });
