angular.module('stow.controllers')

    .controller('OwnerReservationCtrl', function ($rootScope, $state, $scope, $stateParams, Reservation, Picture) {

        if(typeof analytics !== "undefined"){
            analytics.trackView("View Reservation Owner");
        }

        $scope.showOverlay = $scope.loading = true;
        function getReservation() {
            Reservation.getReservation($stateParams.reservationId).then(function (result) {
                $scope.reservation = result;
                console.log($scope.reservation);
                getUserImage();
            }, function (error) {
                analytics.trackException("Failed to retrieve reservation", false);
                console.log(error);
                $rootScope.showMessage(error.message, "invalid");
                $scope.showOverlay = $scope.loading = false;
            });
        }

        function getUserImage() {
            var picId = "";
            if ("profile" in $scope.reservation.renter) {
                picId = $scope.reservation.renter.profile.picId;
            }
            if (picId) {
                Picture.getPicture(picId, picId).then(function (result) {
                    $scope.renterPicture = result;
                }, function (error) {
                    $rootScope.showMessage("Unable to get renter photo", "invalid");
                    console.log(error);
                }).finally(function () {
                    $scope.showOverlay = $scope.loading = false;
                });
            } else {
                $scope.showOverlay = $scope.loading = false;
            }
        }

        $scope.composeMessage = function () {
            analytics.trackEvent("View State", "Messaging renter");
            var blank = {sub: "", body: ""};
            $state.go("app.compose", {
                rentalId: $scope.reservation.rental._id,
                userId: $scope.reservation.renter.id,
                msg: blank
            });
        };

        $scope.acceptRequest = function () {
            $scope.showOverlay = true;
            Reservation.updateReservation($scope.reservation._id, "confirmed", "").then(function (result) {
                analytics.trackEvent("Update Reservation", "Request accepted");
                $rootScope.user.reservations.pending--;
                getReservation();
                $rootScope.showMessage("Reservation Confirmed", "valid");
            }, function (error) {
                analytics.trackException("Failed to accept reservation", false);
                $rootScope.showMessage(error.message, "invalid");
                $scope.showOverlay = false;
            });
        };

        $scope.declineRequest = function () {
            analytics.trackEvent("View State", "Going to decline request");
            $state.go("app.cancel-reservation",
                {
                    title: "Decline Request",
                    reservationId: $scope.reservation._id,
                    username: $scope.reservation.renter.fn
                });
        };

        $scope.cancelReservation = function () {
            analytics.trackEvent("View State", "Going to cancel reservation");
            $state.go("app.cancel-reservation",
                {
                    title: "Cancel Reservation",
                    reservationId: $scope.reservation._id,
                    username: $scope.reservation.renter.fn
                });
        };

        $scope.showRenterDetails = function(status){
          if(status === 'confirmed' || status === 'active') {
              return true;
          } else {
              return false;
          }
        };

        $scope.showFooter = function(status) {
            if(!$scope.showOverlay && (status === 'pending' || status === 'confirmed')){
                return true;
            }else {
                return false;
            }
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
            getReservation();
        });
    });