angular.module('stow.controllers')

    .controller('RenterReservationCtrl', function ($rootScope, $timeout, $scope, $state, $stateParams, $ionicPopup, Reservation, Picture) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("View Reservation Renter");
        }

        $scope.showOverlay = true;
        Reservation.getServiceCharge().then(function (result) {
            $scope.serviceCharge = result;
        }, function (error) {
            console.log(error);
            $rootScope.showMessage("Unable to get service charge", "invalid")
        });

        Reservation.getReservation($stateParams.reservationId).then(function (result) {
            $scope.reservation = result;
            console.log($scope.reservation);
            getSpaceImage();
        }, function (error) {
            analytics.trackException("Failed to retrieve reservation", false);
            console.log(error);
            $rootScope.showMessage(error.message, "invalid")
        });

        function getSpaceImage() {
            var picId = "";
            if ("0" in $scope.reservation.rental.img) {
                picId = $scope.reservation.rental.img[0].id;
            }
            if (picId) {
                Picture.getPicture(picId, picId).then(function (result) {
                    $scope.rentalImage = result;
                }, function (error) {
                    $rootScope.showMessage("Unable to get rental photo", "invalid");
                    console.log(error);
                }).finally(function () {
                    getUserImage();
                });
            } else {
                getUserImage();
            }
        }

        function getUserImage() {
            var picId = "";
            if ("profile" in $scope.reservation.owner) {
                picId = $scope.reservation.owner.profile.picId;
            }
            if (picId) {
                Picture.getPicture(picId, picId).then(function (result) {
                    $scope.ownerImage = result;
                }, function (error) {
                    $rootScope.showMessage("Unable to get owner photo", "invalid");
                    console.log(error);
                }).finally(function () {
                    $scope.showOverlay = $scope.loading = false;
                });
            } else {
                $scope.showOverlay = $scope.loading = false;
            }
        }

        $scope.showTip = function () {
            var alertPopup = $ionicPopup.alert({
                title: 'Stow Service Charge',
                template: 'To offer our services we charge a ' + $scope.serviceCharge + ' service fee on every transaction'
            });
        };

        $scope.composeMessage = function () {
            analytics.trackEvent("View State", "Messaging owner");
            var blank = {sub: "", body: ""};
            $state.go("app.compose", {
                rentalId: $scope.reservation.rental._id,
                userId: $scope.reservation.owner.id,
                msg: blank
            });
        };

        $scope.showOwnerDetails = function () {
            if ($scope.reservation && "current" in $scope.reservation.status) {
                if ($scope.reservation.status.current.type === 'confirmed' || $scope.reservation.status.current.type === 'active') {
                    return true;
                } else {
                    return false;
                }
            }
        };

        $scope.cancelReservation = function () {
            analytics.trackEvent("View State", "Going to cancel reservation");
            $state.go("app.cancel-reservation",
                {
                    title: "Cancel Reservation",
                    reservationId: $scope.reservation._id,
                    username: $scope.reservation.owner.fn
                });
        };

        $scope.showFooter = function(status) {
            if(!$scope.showOverlay && (status === 'pending' || status === 'confirmed')){
                return true;
            }else {
                return false;
            }
        };

        $scope.openMap = function() {
            var address= $scope.reservation.rental.geo.coordinates[1] + "," + $scope.reservation.rental.geo.coordinates[0] ;
            var url='';
            if(device.platform==='iOS'||device.platform==='iPhone'||navigator.userAgent.match(/(iPhone|iPod|iPad)/)){
                url="http://maps.apple.com/maps?q="+encodeURIComponent(address);
            }else if(navigator.userAgent.match(/(Android|BlackBerry|IEMobile)/)){
                url="geo:?q="+encodeURIComponent(address);
            }else{
                //this will be used for browsers if we ever want to convert to a website
                url="http://maps.google.com?q="+encodeURIComponent(address);
            }
            window.open(url, "_system", 'location=no');
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
        });
    });