angular.module('stow.controllers')

    .controller('RequestCtrl', function ($rootScope, $timeout, $scope, $state, $stateParams, $ionicPopup, $ionicSlideBoxDelegate, $ionicHistory, $ionicScrollDelegate, Reservation) {

        if (typeof analytics !== "undefined") {
            analytics.trackView("Request Rental");
        }

        $scope.slide = 0;

        $scope.ableToRequest = function () {
            var ableToRequest = false;
            if ($rootScope.user.payment.hasCreditCard && "src" in $rootScope.user.profilePicture) {
                ableToRequest = true;
            }
            return ableToRequest;
        };

        $scope.init = function () {
            Reservation.getServiceCharge().then(function (result) {
                $scope.serviceCharge = result;
            }, function (error) {
                console.log(error);
                $rootScope.showMessage(error.message, "invalid")
            });
            $ionicSlideBoxDelegate.enableSlide(false);
            $scope.rentalData = $stateParams.rental;
            $scope.reservation = {rentalId: $scope.rentalData._id, checkInDate: "", checkOutDate: "", userComment: ""};
            $scope.pricing = {totalRentalFee: "0", totalServiceCharge: "0", grandTotal: "0"};
        };

        $scope.setTomorrow = function (fieldName) {
            var today = new Date();
            var tomorrow = new Date(today.setDate(today.getDate() + 1));

            if (fieldName === "checkIn" && !$scope.reservation.checkInDate) {
                $scope.reservation.checkInDate = tomorrow;
            } else if (fieldName === "checkOut" && !$scope.reservation.checkOutDate) {
                $scope.reservation.checkOutDate = tomorrow;
            }
        };

        $scope.checkDates = function (fieldName) {
            var validCheckInDate = false,
                validCheckOutDate = false;

            // validate the check-in date if it is not being edited
            if ($scope.reservation.checkInDate) {
                if (validateCheckInDate()) {
                    validCheckInDate = true;

                    // validate the checkout date if it is not being edited
                    if ($scope.reservation.checkOutDate) {
                        if (validateCheckOutDate()) {
                            validCheckOutDate = true;
                        } else {
                            $scope.reservation.checkOutDate = "";
                            resetPricing();
                        }
                    }
                } else {
                    $scope.reservation.checkInDate = "";
                    resetPricing();
                }
            }
            if (validCheckInDate && validCheckOutDate) {
                getEstimate();
            }
        };

        $scope.showTip = function () {
            analytics.trackEvent("Misc", "Viewing service charge popup");
            var alertPopup = $ionicPopup.alert({
                title: 'Stow Service Charge',
                template: 'To offer our services we charge a ' + $scope.serviceCharge + ' service fee on every transaction'
            });
        };

        $scope.sendRequest = function () {
            $scope.showOverlay = true;
            Reservation.requestReservation($scope.reservation).then(function (result) {
                analytics.trackEvent("Send Request", "Success");
                $scope.changeSlide(1);
            }, function (error) {
                analytics.trackException("Failed to request rental", false);
                $rootScope.showMessage(error.message, "invalid")
            }).finally(function () {
                $scope.showOverlay = false;
            });
        };

        $scope.checkBack = function () {
            if ($scope.ableToRequest() && !$rootScope.user.isBackgroundCheckInProgress && ($scope.reservation.checkInDate || $scope.reservation.checkOutDate || $scope.reservation.userComment) && $scope.slide === 0) {
                var confirmPopup = $ionicPopup.confirm({
                    title: 'In Progress',
                    template: "Tap OK to discard reservation. Cancel to continue."
                });
                confirmPopup.then(function (res) {
                    if (res) {
                        $ionicHistory.goBack();
                    }
                });
            } else if ($scope.slide === 2) {
                $scope.changeSlide(0);
            } else {
                $ionicHistory.goBack();
            }
        };

        function validateCheckOutDate() {
            if ($scope.reservation.checkInDate <= $scope.reservation.checkOutDate) {
                return true;
            } else {
                var alertPopup = $ionicPopup.alert({
                    title: 'Invalid Range',
                    template: 'Check-out date must be the same day or after the check-in date.'
                });
                return false;
            }
        }

        function validateCheckInDate() {
            var tomorrow = new Date();
            tomorrow.setDate(tomorrow.getDate() + 1);
            // Time being different will mess up to see if the selected date is today. Reset time fields then compare
            tomorrow.setHours(0, 0, 0, 0);
            // Check in date must be tomorrow or greater
            if ($scope.reservation.checkInDate >= tomorrow) {
                return true;
            } else {
                var alertPopup = $ionicPopup.alert({
                    title: 'Bad Date',
                    template: 'Check-in date must be in the future.'
                });
                return false;
            }
        }

        function resetPricing() {
            $scope.pricing.totalRentalFee = $scope.pricing.totalServiceCharge = $scope.pricing.grandTotal = 0;
        }

        function getEstimate() {
            Reservation.getEstimate($scope.reservation).then(function (result) {
                analytics.trackEvent("Get Estimate", "Success");
                $scope.pricing.totalRentalFee = result.totalRentalFee;
                $scope.pricing.totalServiceCharge = result.totalServiceCharge;
                $scope.pricing.grandTotal = result.grandTotal;
            }, function (error) {
                $rootScope.showMessage(error.message, "invalid");
                analytics.trackException("Failed to get estimate", false);
                console.log(error);
            });
        }

        // Change the slide within the ion-slide-box
        $scope.changeSlide = function (slideNumber) {
            $scope.slide = slideNumber;
            $ionicScrollDelegate.scrollTop();
            $ionicSlideBoxDelegate.slide(slideNumber);
            $ionicSlideBoxDelegate.update();
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            $rootScope.checkSession();
        });
    });