var MessagesService = angular.module('ReservationService', [])

    .service('Reservation', [
        '$window',
        '$http',
        '$rootScope',
        '$q',
        function ($window, $http, $rootScope, $q) {
            return {
                getUserReservations: function () {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/reservations',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            } else if (status == 404) {
                                results.status = 404;
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getReservation: function (reservationId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/reservations/' + reservationId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getEstimate: function (reservation) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/pricing/estimate/?checkInDate=' + reservation.checkInDate + '&checkOutDate=' + reservation.checkOutDate + '&rentalId=' + reservation.rentalId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                getServiceCharge: function () {
                    var deferred = $q.defer();
                    var req = {
                        method: 'GET',
                        url: $rootScope.stowSecureURL + '/pricing/service/',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                requestReservation: function (reservation) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'POST',
                        url: $rootScope.stowSecureURL + '/reservations',
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        },
                        data: reservation
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                updateReservation: function(reservationId, newStatus, reason, comment) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'PUT',
                        url: $rootScope.stowSecureURL + '/reservations/' + reservationId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        },
                        data: {
                            status: newStatus,
                        }
                    };
                    if(reason){
                       req.data.rejectionReason = reason;
                    }
                    if(comment){
                        req.data.userComment = comment;
                    }
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                            deferred.reject(results);
                        });
                    return deferred.promise;
                },
                cancelReservation: function () {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/messages/inbox/' + messageId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                },
                deleteOutboxMessage: function (messageId) {
                    var deferred = $q.defer();
                    var req = {
                        method: 'DELETE',
                        url: $rootScope.stowSecureURL + '/messages/outbox/' + messageId,
                        headers: {
                            'Authorization': 'jwt ' + $rootScope.user.token
                        }
                    };
                    $http(req).
                        success(function (results) {
                            deferred.resolve(results.data);
                        }).
                        error(function (results, status) {
                            deferred.reject(results);
                            if (status == 401) {
                                $rootScope.forceLogout();
                            }
                        });
                    return deferred.promise;
                }
            }
        }]);