angular.module('stow.controllers', [])

    .controller('AppCtrl', function ($rootScope, $scope, $state, $timeout, $ionicHistory, $ionicActionSheet, Authenticate, $ionicSideMenuDelegate, $ionicPopup, $cordovaNetwork) {
        if (typeof analytics !== "undefined") {
            analytics.trackView("Root");
        }

        var environment = "production";

        if (environment === "local") {
            // Local testing
            $rootScope.stowURL = 'http://localhost:3000/api';
            $rootScope.stripeKey = 'pk_test_r9CBoCog3vVMYpDq4PQ2PNqO';
        } else if (environment === "qa") {
            // QA
            $rootScope.stowURL = 'http://qa.stow.io/api';
            $rootScope.stripeKey = 'pk_test_r9CBoCog3vVMYpDq4PQ2PNqO';
        } else if (environment === "production") {
            // Production
            $rootScope.stowURL = 'https://www.stow.io/api';
            $rootScope.stripeKey = 'pk_live_4tLIPwiWvtvN8doTCmEkyZMh';
        }

        $rootScope.stowSecureURL = $rootScope.stowURL + '/secure';
        $rootScope.pictureExtension = ".jpg";
        $rootScope.user = {};
        $rootScope.indoorSpaces = $rootScope.outdoorSpaces = true;
        $rootScope.distanceSlider = {
            value: 50,
            options: {
                floor: 5,
                ceil: 200,
                step: 5,
                hideLimitLabels: true,
                showSelectionBar: true
            }
        };
        $rootScope.priceSlider = {
            minValue: 0,
            maxValue: 200,
            options: {
                floor: 0,
                ceil: 200,
                step: 10,
                hideLimitLabels: true
            }
        };

        // Display system message to user
        $rootScope.showMessage = function (message, type) {
            if ($rootScope.displayMessage !== undefined && "type" in $rootScope.displayMessage && $rootScope.displayMessage.type === 'invalid') {
                $rootScope.displayMessage.shake = true;
                $timeout(function () {
                    $rootScope.displayMessage.shake = false;
                }, 1000);
            }
            $rootScope.displayMessage = {type: type, message: message, visibility: 1};
            $timeout(function () {
                $rootScope.displayMessage = {};
            }, 4000);
        };

        $rootScope.checkSession = function () {
            if ($rootScope.user === undefined || ($rootScope.user !== undefined && !("token" in $rootScope.user))) {
                $state.go('app.login');
                $rootScope.showMessage("Session has expired", "invalid");
            } else {
                return true;
            }
        };

        $rootScope.forceLogout = function () {
            $state.go("app.login");
            analytics.trackEvent('Sessions', 'Force Logout');
        };

        $rootScope.logout = function () {
            // Show the action sheet
            var hideSheet = $ionicActionSheet.show({
                destructiveText: 'Sign Out',
                titleText: 'Are you sure you want to sign out?',
                cancelText: 'Cancel',
                destructiveButtonClicked: function () {
                    $scope.showOverlay = true;
                    hideSheet();
                    $ionicSideMenuDelegate.toggleLeft();
                    Authenticate.logout().then(function () {
                        $state.go('app.login');
                    }, function (error) {
                        $rootScope.showMessage(error.message, "invalid");
                        $rootScope.forceLogout();
                    }).finally(function () {
                        $scope.showOverlay = false;
                    });
                }
            });
        };

        $rootScope.showAlert = function (title, template) {
            var alertPopup = $ionicPopup.alert({
                title: title,
                template: template
            });
        };

        $rootScope.showConfirmation = function (title, template) {
            var confirmPopup = $ionicPopup.confirm({
                title: title,
                template: template
            });
            confirmPopup.then(function (res) {
                if (res) {
                    return true;
                }
            });
        };

        $rootScope.routeCreditCard = function () {
            var route = $rootScope.user.payment.hasCreditCard ? "app.credit-card" : "app.credit-card-edit";
            $state.go(route);
        };

        $rootScope.routeOwnerPayment = function () {
            if (!$rootScope.user.payment.hasManagedAcct) {
                $state.go("app.complete-profile");
            } else if ($rootScope.user.payment.hasBankAcct) {
                $state.go("app.bank-account");
            } else if ($rootScope.user.payment.hasDebitCard) {
                $state.go("app.debit-card");
            } else if (!$rootScope.user.payment.hasDebitCard && !$rootScope.user.payment.hasBankAcct) {
                $state.go("app.debit-card-edit");
            }
        };

        function toUTCDate(date) {
            return new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
        }

        $rootScope.millisToUTCDate = function (date) {
            var millis = new Date(date).getTime();
            return toUTCDate(new Date(millis));
        };

        $scope.$on("$ionicView.beforeEnter", function () {
            if (window.StatusBar) {
                StatusBar.hide();
                ionic.Platform.fullScreen();
            }
        });

        $scope.$on('$ionicView.loaded', function () {
            ionic.Platform.ready(function () {
                //platform is stored so that templates can have access and adjust accordingly
                $rootScope.platform = ionic.Platform.platform();
                // Hide the status bar
                if ($rootScope.platform === "android") {
                    $rootScope.deviceImageDirectory = cordova.file.externalDataDirectory;
                    window.addEventListener("native.keyboardhide", function () {
                        StatusBar.hide();
                        ionic.Platform.fullScreen();
                        window.AndroidFullScreen.immersiveMode(false, false);
                    });
                } else {
                    $rootScope.deviceImageDirectory = cordova.file.dataDirectory;
                }
                // listen for Online event
                $rootScope.$on('$cordovaNetwork:online', function (networkState) {
                    var onlineState = networkState;
                    $rootScope.displayMessage = {};
                });
                // listen for Offline event
                $rootScope.$on('$cordovaNetwork:offline', function () {
                    analytics.trackEvent('Network', 'No Network Connection Detected');
                    $rootScope.displayMessage = {type: 'invalid', message: 'No internet connection', visibility: 1};
                });
            });
        });
    })
    // Directive limits the max number of characters in numeric fields
    .directive('limitTo', [function () {
        return {
            restrict: 'A',
            link: function (scope, elem, attrs) {
                var limit = parseInt(attrs.limitTo);
                angular.element(elem).on('keypress', function (e) {
                    if (this.value.length == limit) e.preventDefault();
                });
            }
        }
    }])
    // Focus an element in the view
    .directive('focusWhen', ['$timeout', function ($timeout) {
        return {
            restrict: 'A',
            require: 'ngModel',
            scope: false,
            link: function (scope, element, attrs, ctrl) {
                scope.$watch(attrs.focusWhen, function (newValue) {
                    if (ctrl.$isEmpty(ctrl.$viewValue) && newValue) {
                        $timeout(function () {
                            element[0].focus();
                        });
                    }
                });
            }
        };
    }])
    .filter('tel', function () {
        return function (tel) {
            if (!tel) {
                return '';
            }

            var value = tel.toString().trim().replace(/^\+/, '');

            if (value.match(/[^0-9]/)) {
                return tel;
            }

            var country, city, number;

            switch (value.length) {
                case 1:
                case 2:
                case 3:
                    city = value;
                    break;

                default:
                    city = value.slice(0, 3);
                    number = value.slice(3);
            }

            if (number) {
                if (number.length > 3) {
                    number = number.slice(0, 3) + '-' + number.slice(3, 7);
                }
                else {
                    number = number;
                }

                return ("(" + city + ") " + number).trim();
            }
            else {
                return "(" + city;
            }

        };
    });